unit Cdopl;

interface

uses WinTypes,
     WinProcs,
     SysUtils,
     Classes,
     Exlib,
     FileToss,
     Messages;

const PlugDie = 1;
      PlugZap = 2;
      PlugInit = 3;

      DoDie   = 13;
      DoZap   = 14;
      DoInit  = 15;

      NoState = 0;

      MAX_BLOCK_SIZE = 254;
      MaxChrResult  = 150;
      MaxRequestProcs  = 100;
      MaxParamsSize = 100;

      Request_Failed  = -1;
      Request_Success  = +2;
      Request_TooLarge = -3;

      Request_Table_Full = -1;
      No_Such_Proc       = -1;

      Request_Flag = 200;
      Response_Flag = 201;

      { Flag to catch and process a file block.}
      CClientOutFileBlocks = 600;
      CHostOutFileBlocks = 607;
      CHostInFileBlocks = 601;
      CClientInFileBlocks = 608;
      CRegisterFile = 602;
      CCancelFile = 603;
      CSendFile = 604;
      CInvalidFileName = 605;
      CGetFile = 606;

      {File transfer cancellation. }
      CFileIsCanceled = 1;
      CFileNotFound = 2;

     type TAssignFile = record  { For exchanging file names. }
      Flag: Word;
      FileName: String[200];
     end;

type TRequestResult = record
      Status: Integer;
      BoolResult: Boolean;
      IntegerResult: LongInt;
      WordResult: Word;
      chrResult: Array[0..MaxChrResult] of Char;
end;

type TRequestProc = procedure ( Sender: TObject;
                                var Params;
                                ParamsLen: Byte;
                                var Return: TRequestResult ) of Object;

type TRequestEntry = record
      Flag: Word;
      Proc: TRequestProc;
end;

type TRequestTable = Array[0..MaxRequestProcs] of TRequestEntry;

type TPacketHeader = record
      PktFlag: Word;
      StructSize: Byte;
      ProcFlag: Word;
end;

const PACKET_HEAD_SIZE = SizeOf( TPacketHeader );

type PRequestPacket  = ^TRequestPacket;
     TRequestPacket = record
      Header: TPacketHeader;
      ProcParams: Array[0..0] of Char;
end;

type PResponsePacket = ^TResponsePacket;
     TResponsePacket = record
      Header: TPacketHeader;
      ProcResult: TRequestResult;
end;

type TClient = class( TObject )
     private
      FLibH: Pointer;
      FPlugState: Word;
      FArguments: String;
      FInitialized: Boolean;
     public
      constructor Create( var LibH: Pointer; var Args: PChar );
      destructor Destroy; override;

      {Client dll entry methods.}
      function Command( var Cmd: Pointer; Len: Word ): Word; virtual;
      function Process: Word; virtual;

      {After the plug has initialized this will be called.
       Create and other objects here.}

      procedure OnInitialize; virtual;
      procedure OnPlugFree; virtual;

      {Utility routines.}
      function SendBlock( var Value; Len: Word ): Boolean;
      function SendCommand( Command: Word ): Boolean;
      procedure Die;
      procedure Zap;

      {Exlib routines.... some of them.}
      function  MDIClient: HWnd;
      function  GetPtr: Pointer;
      procedure AddBlock( Data: Lpvoid; Len: Word );

      property LibH: Pointer
               read FLibH;

      property PlugState: Word
               read FPlugState
               write FPlugState;

      property Arguments: String
               read FArguments
               write FArguments;

      property Initialized: Boolean
               read FInitialized
               write FInitialized;
 end;

 {TCustomClient}
 type TCustomClient = class( TClient )
      private
       FRequestTable: TRequestTable;
       FResponseReady: Boolean;
       FResponse: TRequestResult;
       FFileList: TStringList;
       FOutFileStream: TTossFile;
       FInFileStream: TTossFile;
       FSendFileReady: Boolean;
       FDownLoadDir: String;
       FCurrentSend: String;
       FCurrentRx: String;
      public
       function Command( var Cmd: Pointer; Len: Word ): Word; override;
       function Process: Word; override;

       procedure OnInitialize; override;
       procedure OnPlugFree; override;

       procedure OnData( var Data: Pointer ); virtual;
       procedure OnProcess; virtual;

       function AddRequestProc( RequestProc: TRequestProc; Flag: Word ): Boolean;
       function FindRequestProc( Flag: Word ): Integer;
       function DispatchRequest( var Req: PRequestPacket;
                                 var Res: TResponsePacket ): Boolean;
       function GetEmptyTableEntry: Integer;
       function RequestResponse( ProcFlag: Word; var Params; ParamsLen: Byte ): TRequestResult;
       procedure DoEvent;

         { For file Transfers. }
       procedure AddTransferFile( FileName: String );
       procedure AddPriorityTransferFile( FileName: String );
       procedure GetTransferList( var List: TStrings );
       function CancelSend( FileName: String ): Integer;
       function CancelReceive: Integer;
       function GetClientPath: String;
       function GetClientFileName: String;
       procedure GetHostFile( FileName: String );
       function CancelCurrentSend: Boolean;

       { Mapped to FOutFileStream events. }
       procedure OnSendDone( Sender: TObject ); dynamic;
       procedure OnSendCancel( Sender: TObject ); dynamic;
       procedure OnSendError( Sender: TObject ); dynamic;
       procedure SendOutFileBlock( var What: TFileBlock ); virtual;

       { Mapped to FInFileStream events.}
       procedure OnReceiveDone( Sender: TObject ); dynamic;
       procedure OnReceiveCancel( Sender: TObject ); dynamic;
       procedure OnReceiveError( Sender: TObject ); dynamic;
       procedure SendInFileBlock( var What: TFileBlock ); virtual;

       procedure OnSendProgress( Sender: TObject; Total: LongInt; This: LongInt );dynamic;
       procedure OnReceiveProgress( Sender: TObject; Total: LongInt; This: LongInt ); dynamic;

       property DownLoadDir: String
                read FDownLoadDir
                write FDownLoadDir;

       property CurrentReceiveFile: String
                read FCurrentRx;

       property CurrentSendFile: String
                read FCurrentSend;
end;



implementation

constructor TClient.Create( var LibH: Pointer; var Args: PChar );
begin
 inherited Create;
 FLibH := LibH;
 FArguments := StrPas( Args );
 FInitialized := False;
end;

destructor TClient.Destroy;
begin
 inherited Destroy;
end;


{Client dll entry methods.}
function TClient.Command( var Cmd: Pointer; Len: Word ): Word;
begin
  Result := AL_OK;
  case PWord(Cmd)^ of

   PlugDie: Result := AL_DIE;
   PlugZap: Result := AL_HANGUP;
   PlugInit: begin
              if not FInitialized then OnInitialize;
              FInitialized := True;
              Result := AL_OK;
             end;
  end;
  if not FInitialized then Result := AL_IDLE;
end;

function TClient.Process: Word;
begin
 if not FInitialized then
  begin
   SendCommand( PlugInit );
   Result := AL_IDLE;
   Exit;
  end;
  Result := AL_OK;
 case FPlugState of

  DoDie: begin
          SendCommand( PlugDie );
          FPlugState := NoState;
          Result := AL_IDLE;
         end;

  DoZap: begin
          SendCommand( PlugZap );
          FPlugState := NoState;
          Result := AL_IDLE;
         end;
 end;
end;


procedure TClient.OnInitialize;
begin
 {Do nothing. Meant to be used to create any private objects
  after the dll has been verifies and the plug is out of the
  idle state.}
end;

procedure TClient.OnPlugFree;
begin
 { Used to free any objects before the plug closes.}
end;

{Utility routines.}
function TClient.SendBlock( var Value; Len: Word ): Boolean;
var SPtr: Pointer;
begin
 Result := True;
 if Len <= MAX_BLOCK_SIZE then
  begin
    GetMem( SPtr, Len );
    try
     try
      System.Move( Value, SPtr^, Len );
      AddBlock( SPtr, Len );
     finally
      FreeMem( SPtr, Len );
     end
    except
     {Be Quiet.}
     Result := False;
    end;
  end;
end;

function TClient.SendCommand( Command: Word ): Boolean;
var Cmd: Word;
begin
 Cmd := Command;
 AddBlock( @Cmd, SizeOf(Word) );
end;

procedure TClient.Die;
begin
  FPlugState := DoDie;
end;

procedure TClient.Zap;
begin
 FPlugState := DoZap;
end;



{Exlib routines.... some of them.}
function  TClient.MDIClient: HWnd;
begin
 Result := ExMDIClient( FLibH );
end;

function  TClient.GetPtr: Pointer;
begin
 try
  Result := ExGetPtr( FLibH );
 except
  {Be Quiet.}
 end;
end;

procedure TClient.AddBlock( Data: Lpvoid; Len: Word );
begin
 try
  ExAddBlock( FLibH, Data, Len );
 except
  {Be Quiet.}
 end;
end;

{TCustomClient}
function TCustomClient.Command( var Cmd: Pointer; Len: Word ): Word;
begin
 Result := inherited Command( Cmd, Len );
 if Initialized then OnData( Cmd );
end;

function TCustomClient.Process: Word;

begin
 Result := inherited Process;
 if Initialized then OnProcess;
end;

procedure TCustomClient.OnInitialize;
begin
  inherited OnInitialize;
  FFileList := TStringList.Create;
  FOutFileStream := TTossFile.Create( CClientOutFileBlocks, CHostInFileBlocks );
  FOutFileStream.OnDone := OnSendDone;
  FOutFileStream.OnCancel := OnSendCancel;
  FOutFileStream.OnError := OnSendError;
  FOutFileStream.OnProgress := OnSendProgress;
  FOutFileStream.OnSend := SendOutFileBlock;
  FInFileStream := TTossFile.Create( CClientInFileBlocks, CHostOutFileBlocks );
  FInFileStream.OnDone := OnReceiveDone;
  FInFileStream.OnCancel := OnReceiveCancel;
  FInFileStream.OnError := OnReceiveError;
  FInFileStream.OnProgress := OnReceiveProgress;
  FInFileStream.OnSend := SendInFileBlock;
  FInFileStream.SaveAsName := '';
  FOutFileStream.SaveAsName := '';
  FDownLoadDir := GetClientPath;
  FSendFileReady := True;
end;

procedure TCustomClient.OnPlugFree;
begin
 if FOutFileStream.SaveAsName <> '' then FOutFileStream.Cancel;
 if FInFileStream.SaveAsName <> '' then FInFileStream.Cancel;
 if Assigned(FFileList) then FFileList.Free;
 if Assigned(FOutFileStream) then FOutFileStream.Free;
 if Assigned(FInFileStream) then FInFileStream.Free;
 inherited OnPlugFree;
end;

procedure TCustomClient.OnData( var Data: Pointer );
var Resp: TResponsePacket;
    Block: TFileBlock;
    RegFile: TAssignFile;
    TempStr: String;
begin
 case PWord(Data)^ of
  Request_Flag: begin
                 DispatchRequest(PRequestPacket(Data), Resp );
                 SendBlock( Resp, SizeOf( TResponsePacket ) );
                end;

  Response_Flag: begin
                 System.Move( PResponsePacket(Data)^.ProcResult,
                              FResponse, SizeOf( TRequestResult ));
                 FResponseReady := True;
                end;

  CClientOutFileBlocks: begin
                   System.Move( Data^, Block, SizeOf( TFileBlock ) );
                   FOutFileStream.Catch( Block );
                  end;

   CClientInFileBlocks: begin
                   System.Move( Data^, Block, SizeOf( TFileBlock ) );
                   FInFileStream.Catch( Block );
                  end;

   CRegisterFile: begin
                    System.Move( Data^, RegFile, SizeOf( TAssignFile ) );
                    FInFileStream.SaveAsName := DownloadDir + ExtractFileName(RegFile.FileName);
                    FCurrentRx := ExtractFileName(RegFile.FileName);
                    RegFile.Flag := CSendFile;
                    SendBlock( RegFile, SizeOf( TAssignFile ));
                   end;

    CGetFile: begin
               System.Move( Data^, RegFile, SizeOf( TAssignFile ) );
               if FileExists( DownloadDir + RegFile.FileName ) then
                begin
                  AddTransferFile( DownloadDir + ExtractFileName( RegFile.FileName) );
                  FCurrentSend := RegFile.FileName;
                end
               else
                 begin
                  RegFile.Flag := CInvalidFileName;
                  SendBlock( RegFile, SizeOf( RegFile ) );
                 end;
              end;

    CInvalidFileName: begin
                       System.Move( Data^, RegFile, SizeOf( TAssignFile ) );
                       CancelReceive;
                      end;

    CSendFile: begin
                TempStr := FCurrentSend;
                FOutFileStream.SendFile( FCurrentSend, 1 );
                FCurrentSend := TempStr;
               end;

 end;
end;

procedure TCustomClient.OnProcess;
var RegFile: TAssignFile;
begin
 if (FSendFileReady) and
    (FFileList.Count > 0) then
 begin
  if FileExists( FFileList.Strings[0] ) then
   begin
    FSendFileReady := False;
    FOutFileStream.SaveAsName := FFileList.Strings[0];
    FCurrentSend := FOutFileStream.SaveAsName;
    RegFile.Flag := CRegisterFile;
    RegFile.FileName := ExtractFileName( FFileList.Strings[0] );
    FFileList.Delete(0);
    SendBlock( RegFile, SizeOf( TAssignFile ) );
   end;
 end;
end;


function TCustomClient.AddRequestProc( RequestProc: TRequestProc; Flag: Word ): Boolean;
var TableIndex: Integer;
begin
 Result := False;
 TableIndex := GetEmptyTableEntry;
 if TableIndex <> Request_Table_Full then
 begin
  FRequestTable[TableIndex].Flag := Flag;
  FRequestTable[TableIndex].Proc := RequestProc;
  Result := True;
 end;
end;

function TCustomClient.FindRequestProc( Flag: Word ): Integer;
var i: Integer;
begin
 Result := No_Such_Proc;
 for i := 0 to MaxRequestProcs do
  if FRequestTable[i].Flag = Flag then
  begin
   Result := i;
   Break;
  end;
end;

function TCustomClient.DispatchRequest( var Req: PRequestPacket;
                                        var Res: TResponsePacket ): Boolean;
var ProcIndex: Integer;
    Buff: Array[0..MaxParamsSize] of Char;
begin
 Res.Header.PktFlag := Response_Flag;
 Res.Header.StructSize := SizeOf( TResponsePacket );
 Res.ProcResult.Status := Request_Failed;
 ProcIndex := FindRequestProc( Req^.Header.ProcFlag );
 if ProcIndex <> No_Such_Proc then
 begin
  System.Move( Req^.ProcParams, Buff, Req^.Header.StructSize - PACKET_HEAD_SIZE );
  FRequestTable[ProcIndex].Proc( Self,
                                 Buff,
                                 Req^.Header.StructSize - PACKET_HEAD_SIZE,
                                 Res.ProcResult );
 end;
end;


function TCustomClient.GetEmptyTableEntry: Integer;
var i: Integer;
begin
 Result := Request_Table_Full;
 for i := 0 to MaxRequestProcs do
  if not Assigned(FRequestTable[i].Proc) then
  begin
   Result := i;
   Break;
  end;
end;

function TCustomClient.RequestResponse( ProcFlag: Word;
                                        var Params;
                                        ParamsLen: Byte ): TRequestResult;
var RequestPacket: PRequestPacket;
    TotalSize: Byte;
begin

 if ParamsLen > MaxParamsSize then
 begin
  Result.Status := Request_TooLarge;
  Exit;
 end;

 try
  FResponseReady := False;
  TotalSize := PACKET_HEAD_SIZE + ParamsLen;
  GetMem( RequestPacket, TotalSize );
  RequestPacket^.Header.PktFlag := Request_Flag;
  RequestPacket^.Header.ProcFlag := ProcFlag;
  RequestPacket^.Header.StructSize := TotalSize;
  System.Move( Params, RequestPacket^.ProcParams, ParamsLen );
  SendBlock( RequestPacket^, TotalSize );
  FreeMem( RequestPacket, TotalSize );
 except
  Result.Status := Request_Failed;
 end;

{ Loop until answer. }
 while not FResponseReady do DoEvent;
 System.Move( FResponse, Result, SizeOf( TRequestResult ) );

end;

procedure TCustomClient.DoEvent;
var Msg: TMsg;
begin
  if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then
  begin
    if Msg.Message <> WM_QUIT then
    begin
     TranslateMessage(Msg);
     DispatchMessage(Msg);
    end
    else Die;
  end;
end;

{ File Transfer Events/Procs. }
 procedure TCustomClient.AddTransferFile( FileName: String );
 begin
  FFileList.Add( DownloadDir + ExtractFileName(FileName) );
 end;

 procedure TCustomClient.AddPriorityTransferFile( FileName: String );
 begin
  if FFileList.Count > 0 then
   FFileList.Insert( 0, 'Client.AddTransfer:' + DownloadDir + ExtractFileName(FileName) )
  else
   FFileList.Add( DownloadDir + ExtractFileName(FileName) );
 end;

 procedure TCustomClient.GetTransferList( var List: TStrings );
 begin
  List.Assign( FFileList );
 end;

 function TCustomClient.CancelSend( FileName: String ): Integer;
 var Idx: Integer;
 begin
  Result := CFileIsCanceled;
  if FOutFileStream.SaveAsName = FileName then
     FOutFileStream.Cancel
  else
     begin
      if FFileList.Find( FileName, Idx ) then
         FFileList.Delete( Idx )
      else Result := CFileNotFound;
     end;
 end;

 function TCustomClient.CancelCurrentSend: Boolean;
 begin
  FOutFileStream.Cancel;
 end;

 function TCustomClient.CancelReceive: Integer;
 begin
  Result := CFileIsCanceled;
  if FInFileStream.SaveAsName <> '' then
     FInFileStream.Cancel
  else Result := CFileNotFound;
 end;

 function TCustomClient.GetClientPath: String;
 var chrPath: Array[0..255] of Char;
     Success: Integer;
 begin
  Result := '';
  Success := GetModuleFileName( HInstance, chrPath, 255 );
  if Success > 0 then Result := ExtractFilePath(StrPas( chrPath ));
 end;

 function TCustomClient.GetClientFileName: String;
 var chrPath: Array[0..255] of Char;
     Success: Integer;
 begin
  Result := '';
  Success := GetModuleFileName( HInstance, chrPath, 255 );
  if Success > 0 then Result := StrPas( chrPath );
 end;

 { Mapped to FOutFileStream events. }
 procedure TCustomClient.OnSendDone( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
  FCurrentSend := '';
 end;

 procedure TCustomClient.OnSendCancel( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
  FCurrentSend := '';
 end;

 procedure TCustomClient.OnSendError( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
  FCurrentSend := '';
  { Do nothing. }
 end;

 procedure TCustomClient.GetHostFile( FileName: String );
 var ReqFile: TAssignFile;
 begin
  ReqFile.Flag := CGetFile;
  ReqFile.FileName := ExtractFileName( FileName );
  SendBlock( ReqFile, SizeOf( TAssignFile ) );
 end;

 { Mapped to FInFileStream events.}
 procedure TCustomClient.OnReceiveDone( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
  FCurrentRx := '';
 end;

 procedure TCustomClient.OnReceiveCancel( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
  FCurrentRx := '';
 end;

 procedure TCustomClient.OnReceiveError( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
  FCurrentRx := '';
 end;

procedure TCustomClient.OnSendProgress( Sender: TObject; Total: LongInt; This: LongInt );
begin
 {}
end;

procedure TCustomClient.OnReceiveProgress( Sender: TObject; Total: LongInt; This: LongInt );
begin
 {}
end;

procedure TCustomClient.SendOutFileBlock( var What: TFileBlock );
begin
 SendBlock( What, SizeOf( TFileBlock ) );
end;

procedure TCustomClient.SendInFileBlock( var What: TFileBlock );
begin
 SendBlock( What, SizeOf( TFileBlock ) );
end;

end.
