unit Dopl;

{ Delphi Object Programmers Library.
  Copyright 1996 Argon D. Helm.
  Based on source provided by Excalibur Communications Inc.}

interface

 uses WinTypes,
      WinProcs,
      SysUtils,
      Alchemw,
      Classes,
      Ver;


const UserListCapacity = 255;          {Preset user list capacity at 255 users.}
      MaxBlockLength   = 254;{Maximum size of an Excalibur Data Transfer Block.}
      PlugDie          = 1;
      PlugZap          = 2;
      PlugInit         = 3;
      PlugPointer      = 4;
      PlugPtrReg       = 5;

type LPSLOT = Pointer;

type TPlugType = (ptGraphic,ptHidden,ptClient,ptGraphicMulti,ptHiddenMulti,
                  ptClientMulti);


type PPointerHeader = ^TPointerHeader;      {Descriptor for sending large blks.}
     TPointerHeader = record
      ThisSize: Word;
      TotalSize: Word;
end;

type TAssignFile = record  { For exchanging file names. }
      Flag: Word;
      FileName: String[200];
     end;

{ Encapsulates basic Host side plugin actions.
  And acts as a conduit to the alchemist workbench API. }

type TPlugin = class( TObject )
      private
	FMultiLingual: Boolean;	  { Plugin uses standard multi-lingual setup. }
        FWhoIsHere: TList;        { List of who is in the plugin. }
	FSlot: LPSLOT;            { Keep track of the slot pointer. }
        FDllName: String;         { Title of Plugin = Name of Client Dll. (Host Client) }
        FOtherFiles: TStrings;    { For dl'ing multiple files on new plug. }
        FPlugType: TPlugType;     { What type of plug is this? AL_WIN_PLUGIN etc. }
        FDoList: Boolean;         { Want me to compile a user list? }
        FOtherFilesSent: Integer; { Have all files been sent? Yes when equals other files count. }
        FDllSent: Boolean;        { Has the plugin been sent? }
        FWindowTitle: String;     { For graphic plug caption. }
        FArguments: String;       { String to send to client PlugInit. }
        FMultiUser: Boolean;      { Can more than one person use this plug?}
        FInitialized: Boolean;
        
      protected
	procedure BroadAllHere( var Data: Pointer ); dynamic;
	procedure BroadEnter( var Data: Pointer ); dynamic;
	procedure BroadLeave( var Data: Pointer ); dynamic;
        procedure SetSlot( Slot: LPSLOT );
        procedure InitLanguage; virtual;
        function  GetUserCount: Word; virtual;
        function  GetUserFromList( Index: Integer ): tagAlBroadEnter; dynamic;
        procedure SetOtherFiles( TheOtherFiles: TStrings );
        procedure Die;
        procedure Zap;

        property Slot: LPVOID
                 read FSlot;

      public
	constructor Create( var Slot: LPSLOT );
        destructor Destroy; override;

        procedure LoadOtherFilesFromFile( FileName: String );


        {Utility send routines.}
        function SendBlock( var Block; Len: Word ): Boolean;
        function SendCommand( Command: Word ): Boolean;
        procedure BeforePlugInit; virtual;

        { Shells for the entry points. }
        function PlgType( var Title: PChar ): Word; virtual;
	function Process: Word; virtual;
	function Dialog( Return: Word ): Word; virtual;
	function GrailData( var Data: Pointer; More: Bool ): Bool; virtual;
	function Mouse( Code: Byte; X,Y: Word ): Word; virtual;
	function Button( Id: Word ): Word; virtual;
	function DLLSize: LongInt; virtual;
        function Unique( args, cMessage: PChar ): Bool; virtual;

	function NextDLL( var Name: PChar; NameLimit: Word;
                          var Dest: PChar; DestLimit: Word ):Bool; virtual;

	function CheckDLL( CurOS: LongInt;
                           ClientDllInfo: Pointer;
 		           var Company: PChar; var Intern: PChar;
                           var Ver: PChar; var Description: PChar ): Word; virtual;

	function Command( var Cmd: Pointer; Len: Word ): Word; virtual;
	function Broadcast( var Data: Pointer; Len: Word ): Word; virtual;

	function Receive( var Data: Pointer; Len: Word;
                                        Sender: LongInt ): Word; virtual;

	procedure Activate( Active: Boolean ); virtual;
	procedure Paint( var Dc: HDC; Erase: Bool; Bound: PRect ); virtual;
	procedure Resize( Width: Word; Height: Word ); virtual;


        property UserCount: Word
                 read GetUserCount;

        property OtherFiles: TStrings
                 read FOtherFiles
                 write SetOtherFiles;

        property ClientDllName: String
                 read FDllName
                 write FDllName;

        property WindowTitle: String
                 read FWindowTitle
                 write FWindowTitle;

        property ClientArgs: String
                 read FArguments
                 write FArguments;

        property MultiLingual: Boolean
                 read FMultiLingual
                 write FMultiLingual;

        property BuildUserList: Boolean
                 read FDoList
                 write FDoList;

        property PlugType: TPlugType
                 read FPlugType
                 write FPlugType;


        property DllSent: Boolean
                 read FDllSent;

        property Initialized: Boolean
                 read FInitialized
                 write FInitialized;

	{ Shells for library functions }
          function  AlSlot:Word;
          function  AlArgs( dest: PChar; len: Word ):PChar;
          procedure AlCaption( title: PChar );
          procedure AlLogWrite( text: PChar );
          function  AlAlloc( size: LongInt ):HWND;
          procedure AlFree;
          function  AlLock: LPVOID;
          procedure AlUnlock;
          function  AlReAlloc( size: LongInt ):HWND;
          function  AlGetHandle:HWND;
          procedure AlSetHandle(dHandle: HWND );
          function  AlGetPointer:LPVOID;
          procedure AlSetPointer( userPtr: LPVOID );
          procedure AlAddBlock( data: LPVOID; len: Word );
          function  AlMainWindow:HWND;
          procedure AlSetWindowOrg( hdc: HDC );
          function  AlSetState( value: Word ):Word;
          function  AlGetState:Word;
          procedure AlPushState( value: Word );
          function  AlPopState:Word;

          { -- screen handling routines }
          function  AlOpenScreen( name: PChar; ofs: OFSTRUCT; mode: Word ): Integer;
          function  AlOpenScratch(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
          function  AlOpenUserFile(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
          function  AlOpenGlobalFile(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
          function  AlDateTimeNow:LongInt;
          procedure AlDateDecode( date: LongInt; var year, month, day: Word );
          function  AlDateEncode( year, month, day: Word ):LongInt;
          procedure AlReleaseButton;
          procedure AlSound(  sound: PChar );
          procedure AlMessageBox( TheMessage: PChar; title: PChar; code: Word );
          procedure AlMouseDisable;
          procedure AlMouseEnable;
          procedure AlMouseRelease;

          { -- Dialog processing functions }
          procedure AlExecDialog;
          function  AlLoadDialog( resourceID: PChar; module: HWND ):Bool;
          procedure AlNewDialog( caption: PChar; x, y, w, h: Word );
          procedure AlDialogPress( id: Word );
          procedure AlSetString( id: Word; data: PChar );
          procedure AlSetCheck( id: Word );
          procedure AlLbAddString( id: Word; data: PChar );
          procedure AlCbAddString( id: Word; data: PChar );
          procedure AlLimitText( id, len: Word );
          procedure AlItemEdit( id, x, y, w, h: Word; style: LongInt );
          procedure AlItemStatic(  id, x, y, w, h: Word; style: LongInt; text: PChar );
          procedure AlItemButton(  id, x, y, w, h: Word; style: LongInt; text: PChar );
          procedure AlItemListBox(  id, x, y, w, h: Word; style: LongInt );
          procedure AlItemComboBox(  id, x, y, w, h: Word; style: LongInt );
          procedure AlItemScrollBar(  id, x, y, w, h: Word; style: LongInt );
          procedure AlItemBorShade(  id, x, y, w, h: Word; style: LongInt; text: PChar );
          procedure AlItemBorCheck(  id, x, y, w, h: Word; style: LongInt; text: PChar );
          procedure AlItemBorRadio(  id, x, y, w, h: Word; style: LongInt; text: PChar );
          procedure AlItemBorButton(  id, x, y, w, h: Word; style: LongInt );
          function  AlDialogReturn:Word;
          function  AlGetText(  id: Word; dest: PChar; len: Word ):PChar;
          function  AlGetTextLen(  id: Word ):Integer;
          function  AlGetCheck(  id: Word ):Bool;
          function  AlGetSelected(  id: Word ):Word;
          function  AlGetSelList(  id: Word; dest: Word; limit: Integer ):Word;
          function  AlGetSelCount(  id: Word ):Word;
          function  AlGetScrollPos(  id: Word ):Word;
          procedure AlSetScrollPos(  id, pos: Word );
          procedure AlSetScrollRange(  id, lower, upper: Word );
          procedure AlLbSelect(  id, pick: Word );
          procedure AlLbSelectItem(  id, pick: Word );
          procedure AlLbSelectRange(  id, lower, upper: Word );
          procedure AlCbSelect(  id, pick: Word );

          { -- general Grail database access }
          function  AlGrailRequest(  table, fields, filter: PChar ):Bool;
          function  AlGrailExecute(  command: PChar ):Bool;
          function  AlCreateGrailInsert(  table: PChar; size: Word; needID: Bool ):Bool;
          function  AlCreateGrailUpdate(  table: PChar; size: Word; id: LongInt ):Bool;
          function  AlGrailLock:LPVOID;
          procedure AlGrailUnlock;
          function  AlGrailLastInsert:LongInt;

          { -- Current User Information }
          function  AlGetUserID:LongInt;
          function  AlGetUserULBytes:LongInt;
          function  AlGetUserDLBytes:LongInt;
          function  AlGetUserBalance:LongInt;
          function  AlGetUserRevenue:LongInt;
          function  AlGetUserLastCall:LongInt;
          function  AlGetUserDayByteLeft:LongInt;
          function  AlGetUserWeekByteLeft:LongInt;
          function  AlGetUserMonthByteLeft:LongInt;
          function  AlGetUserPermByte:LongInt;
          function  AlGetUserPermTime:LongInt;
          function  AlGetUserFirstCall:LongInt;
          function  AlGetUserExpiration:LongInt;
          function  AlGetUserCallByte:LongInt;
          function  AlGetUserDayByte:LongInt;
          function  AlGetUserWeekByte:LongInt;
          function  AlGetUserMonthByte:LongInt;
          function  AlGetUserBirth:LongInt;
          function  AlGetUserOtherMail:LongInt;
          function  AlGetUserULFiles:Word;
          function  AlGetUserDLFiles:Word;
          function  AlGetUserTotalCall:Word;
          function  AlGetUserDayTimeLeft:Word;
          function  AlGetUserWeekTimeLeft:Word;
          function  AlGetUserMonthTimeLeft:Word;
          function  AlGetUserByteRatio:Word;
          function  AlGetUserFileRatio:Word;
          function  AlGetUserCallTime:Word;
          function  AlGetUserDayTime:Word;
          function  AlGetUserWeekTime:Word;
          function  AlGetUserMonthTime:Word;
          function  AlGetUserLanguage:Word;
          function  AlGetUserFirst(  dest: PChar; len: Word ):PChar;
          function  AlGetUserMiddle(  dest: PChar; len: Word ):PChar;
          function  AlGetUserLast(  dest: PChar; len: Word ):PChar;
          function  AlGetUserHandle(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPassword(  dest: PChar; len: Word ):PChar;
          function  AlGetUserTitle(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPosition(  dest: PChar; len: Word ):PChar;
          function  AlGetUserCompany(  dest: PChar; len: Word ):PChar;
          function  AlGetUserAddress1(  dest: PChar; len: Word ):PChar;
          function  AlGetUserAddress2(  dest: PChar; len: Word ):PChar;
          function  AlGetUserCity(  dest: PChar; len: Word ):PChar;
          function  AlGetUserState(  dest: PChar; len: Word ):PChar;
          function  AlGetUserCountry(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPostal(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPhoneHome(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPhoneWork(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPhoneFax(  dest: PChar; len: Word ):PChar;
          function  AlGetUserPhoneBBS(  dest: PChar; len: Word ):PChar;
          function  AlGetUserFirstVerify(  dest: PChar; len: Word ):PChar;
          function  AlGetUserLastVerify(  dest: PChar; len: Word ):PChar;
          function  AlGetUserLocalNet(  dest: PChar; len: Word ):PChar;
          function  AlGetUserNetID(  dest: PChar; len: Word ):PChar;
         {function  AlGetUserComment(  dest: PChar; len: Word ):PChar;}
          function  AlGetUserFlag(  dest: PChar; index: Word ):Bool;
          function  AlGetUserFlagCount:Word;
          function  AlIsUserFlag(  flag: PChar ):Bool;
          procedure AlSetUserFlag(  name: PChar );
          procedure AlClearUserFlag(  name: PChar );
          procedure AlSetUserULBytes(  value: LongInt );
          procedure AlSetUserDLBytes(  value: LongInt );
          procedure AlSetUserBalance(  value: LongInt );
          procedure AlSetUserRevenue(  value: LongInt );
          procedure AlSetUserLastCall(  value: LongInt );
          procedure AlSetUserDayByteLeft(  value: LongInt );
          procedure AlSetUserWeekByteLeft(  value: LongInt );
          procedure AlSetUserMonthByteLeft(  value: LongInt );
          procedure AlSetUserPermByte(  value: LongInt );
          procedure AlSetUserPermTime(  value: LongInt );
          procedure AlSetUserFirstCall(  value: LongInt );
          procedure AlSetUserExpiration(  value: LongInt );
          procedure AlSetUserCallByte(  value: LongInt );
          procedure AlSetUserDayByte(  value: LongInt );
          procedure AlSetUserWeekByte(  value: LongInt );
          procedure AlSetUserMonthByte(  value: LongInt );
          procedure AlSetUserBirth(  value: LongInt );
          procedure AlSetUserOtherMail(  value: LongInt );
          procedure AlSetUserULFiles(  value: Word );
          procedure AlSetUserDLFiles(  value: Word );
          procedure AlSetUserTotalCall(  value: Word );
          procedure AlSetUserDayTimeLeft(  value: Word );
          procedure AlSetUserWeekTimeLeft(  value: Word );
          procedure AlSetUserMonthTimeLeft(  value: Word );
          procedure AlSetUserByteRatio(  value: Word );
          procedure AlSetUserFileRatio(  value: Word );
          procedure AlSetUserCallTime(  value: Word );
          procedure AlSetUserDayTime(  value: Word );
          procedure AlSetUserWeekTime(  value: Word );
          procedure AlSetUserMonthTime(  value: Word );
          procedure AlSetUserLanguage(  value: Word );
          function  AlSetUserFirst(  data: PChar ):PChar;
          function  AlSetUserMiddle(  data: PChar ):PChar;
          function  AlSetUserLast(  data: PChar ):PChar;
          function  AlSetUserHandle(  data: PChar ):PChar;
          function  AlSetUserPassword(  data: PChar ):PChar;
          function  AlSetUserTitle(  data: PChar ):PChar;
          function  AlSetUserPosition(  data: PChar ):PChar;
          function  AlSetUserCompany(  data: PChar ):PChar;
          function  AlSetUserAddress1(  data: PChar ):PChar;
          function  AlSetUserAddress2(  data: PChar ):PChar;
          function  AlSetUserCity(  data: PChar ):PChar;
          function  AlSetUserState(  data: PChar ):PChar;
          function  AlSetUserCountry(  data: PChar ):PChar;
          function  AlSetUserPostal(  data: PChar ):PChar;
          function  AlSetUserPhoneHome(  data: PChar ):PChar;
          function  AlSetUserPhoneWork(  data: PChar ):PChar;
          function  AlSetUserPhoneFax(  data: PChar ):PChar;
          function  AlSetUserPhoneBBS(  data: PChar ):PChar;
          function  AlSetUserFirstVerify(  data: PChar ):PChar;
          function  AlSetUserLastVerify(  data: PChar ):PChar;
          function  AlSetUserLocalNet(  data: PChar ):PChar;
          function  AlSetUserNetID(  data: PChar ):PChar;
          {function  AlSetUserComment(  data: PChar ):PChar;}

          { -- Current Port Information }
          function  AlGetPortID:LongInt;
          function  AlGetPortLastUser:LongInt;
          function  AlGetPortCalls:LongInt;
          function  AlGetPortBaud:LongInt;
          function  AlGetPortTemplate:LongInt;
          function  AlGetPortGuest:LongInt;
          function  AlGetPortWinStyle:Word;
          function  AlGetPortWinX:Word;
          function  AlGetPortWinY:Word;
          function  AlGetPortWinW:Word;
          function  AlGetPortWinH:Word;
          function  AlGetPortBeepRepeat:Word;
          function  AlGetPortBeepInterval:Word;
          function  AlGetPortPort:BYTE;
          function  AlGetPortRingDetect:Bool;
          function  AlGetPortUseHangup:Bool;
          function  AlGetPortRingCount:BYTE;
          function  AlGetPortTimeout:BYTE;
          function  AlGetPortLockRate:Bool;
          procedure AlGetPortDriveMap(  dest: PChar );
          function  AlGetPortNetName(  dest: PChar; len: Word ):PChar;
          function  AlGetPortSendExcal(  dest: PChar; len: Word ):PChar;
          function  AlGetPortLogin(  dest: PChar; len: Word ):PChar;
          function  AlGetPortNewUser(  dest: PChar; len: Word ):PChar;
          function  AlGetPortModemName(  dest: PChar; len: Word ):PChar;
          function  AlGetPortReset(  dest: PChar; len: Word ):PChar;
          function  AlGetPortInit(  dest: PChar; len: Word ):PChar;
          function  AlGetPortOffHook(  dest: PChar; len: Word ):PChar;
          function  AlGetPortOnHook(  dest: PChar; len: Word ):PChar;
          function  AlGetPortAnswer(  dest: PChar; len: Word ):PChar;
          function  AlGetPortRing(  dest: PChar; len: Word ):PChar;
          function  AlGetPortCIDPrefix(  dest: PChar; len: Word ):PChar;
          function  AlGetPortCIDPrivate(  dest: PChar; len: Word ):PChar;
          function  AlGetPortCIDNone(  dest: PChar; len: Word ):PChar;
          function  AlGetPortHangup(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect1200(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect2400(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect4800(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect9600(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect19k(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect38k(  dest: PChar; len: Word ):PChar;
          function  AlGetPortConnect57k(  dest: PChar; len: Word ):PChar;
          function  AlGetPortErrorCorrect(  index: Word; dest: PChar; len: Word ):PChar;
          function  AlGetPortEffBaud(  index: Word; dest: PChar; len: Word ):PChar;
          function  AlGetPortWaitMessage(  dest: PChar; len: Word ):PChar;
          function  AlGetPortScratch(  dest: PChar; len: Word ):PChar;
          function  AlGetPortSoundName(  dest: PChar; len: Word ):PChar;
          function  AlGetPortFirst(  dest: PChar; len: Word ):PChar;
          function  AlGetPortMiddle(  dest: PChar; len: Word ):PChar;
          function  AlGetPortLast(  dest: PChar; len: Word ):PChar;
          function  AlGetPortHandle(  dest: PChar; len: Word ):PChar;

          { -- system settings }
          function  AlGetSysMajorRev:Word;
          function  AlGetSysMinorRev:Word;
          function  AlGetSysPageFrom:Word;
          function  AlGetSysPageTo:Word;
          function  AlGetSysByteCreditPct:Word;
          function  AlGetSysDayCarryPct:Word;
          function  AlGetSysWeekCarryPct:Word;
          function  AlGetSysMonthCarryPct:Word;
          function  AlGetSysDayCarryLimit:LongInt;
          function  AlGetSysWeekCarryLimit:LongInt;
          function  AlGetSysMonthCarryLimit:LongInt;
          function  AlGetSysCanPage:Bool;
          function  AlGetSysAdultAge:BYTE;
          function  AlGetSysLogClear:Bool;
          function  AlGetSysPageClear:Bool;
          function  AlGetSysMinimumTime:BYTE;
          function  AlGetSysChargePartial:Bool;
          function  AlGetSysNoULClock:Bool;
          function  AlGetSysCreditNow:Bool;
          function  AlGetSysSystemName(  dest: PChar; len: Word ):PChar;
          function  AlGetSysSerial(  dest: PChar; len: Word ):PChar;
          function  AlGetSysCreditCard(  dest: PChar; len: Word ):PChar;
          function  AlGetSysDefTopic(  dest: PChar; len: Word ):PChar;
          function  AlGetSysTimeZone(  dest: PChar; len: Word ):PChar;
          function  AlGetSysChat(  index: Word; dest: PChar; len: Word ):PChar;
          function  AlGetSysScreenDir(  dest: PChar; len: Word ):PChar;
          procedure AlGetSysLogFlags(  dest: PChar );
          function  AlGetSysUserDir(  dest: PChar; len: Word ):PChar;
          function  AlGetSysPreviewDir(  dest: PChar; len: Word ):PChar;
          function  AlGetSysNoAttachments:Bool;
          function  AlGetSysPreviewCharge:Word;
          function  AlGetSysHandles:Bool;

          { -- Graphic commands }
          procedure AlScreenStart(  color: BYTE );
          procedure AlScreenEnd;
          procedure AlLine(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
          procedure AlRectangle(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
          procedure AlEllipse(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
          procedure AlFRectangle(  x1, y1, x2, y2: Word; color, mode: BYTE;
                                  width: Word; fg, bg, fill: BYTE; transparent: Bool );

          procedure AlFEllipse(  x1, y1, x2, y2: Word; color, mode: BYTE;
                                width: Word; fg, bg, fill: BYTE; transparent: Bool );

          procedure AlRoundRect(  x1, y1, x2, y2: Word; xe, ye: Word;
                                 color, mode: BYTE; width: Word );

          procedure AlFRoundRect(  x1, y1, x2, y2: Word; xe, ye: Word;
                                  color, mode: BYTE; width: Word; fg, bg, fill: BYTE;
                                                                  transparent: Bool );

          procedure AlArc(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                                                  color, mode: BYTE; width: Word );

          procedure AlPie(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                                                              color, mode: BYTE;
                                    width: Word; fg, bg, fill: BYTE; transparent: Bool );

          procedure AlChord(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                             color, mode: BYTE; width: Word; fg, bg, fill: BYTE;
                             transparent: Bool );

          procedure AlHit(  x1, y1, x2, y2: Word; id, key: BYTE  );

          procedure AlButton(  x1, y1, x2, y2: Word;
                               id, key, font, width, height, style,
                              color: BYTE; caption: PChar );

          procedure AlPolygon(  count: Word; var point: Word;
                               color, mode: BYTE; width: Word );

          procedure AlFPolygon(  count: Word; var point: Word;
                                color, mode: BYTE; width: Word;
                                fg, bg, fill: BYTE; transparent: Bool );

          procedure AlText(  x1, y1, x2, y2: Word;
                            font, width, height, style, position,
                            color: BYTE; text: PChar );

          procedure AlCard(  card: BYTE; x1, y1, x2, y2: Word );

          { -- International conversions }
          function  AlUserMoneyFormat(  dest: PChar; amt: LongInt ):PChar;
          function  AlSysMoneyFormat(  dest: PChar; amt: LongInt ):PChar;
          function  AlSysShortDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
          function  AlSysLongDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
          function  AlUserShortDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
          function  AlUserLongDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
          function  AlSysTime(  date: LongInt; dest: PChar; len: Integer ): PChar;
          {function  AlUserTime(  date: LongInt; dest: PChar; len: Integer ): PChar;
           function  AlSysDateScan(  var dest: LongInt; date: PChar ):Bool;
           function  AlUserDateScan(  var dest: LongInt; date: PChar ):Bool;
           function  AlSysTimeScan(  var dest: LongInt; time: PChar ):Bool;
           function  AlUserTimeScan(  var dest: LongInt; time: PChar ):Bool;
           procedure AlUserInternational( sShortDate, sLongDate, s1159, s2359, sTime: PChar;
           var iTime, iTLZero: Bool; sThousand, sDecimal, sList, sDate: PChar );}

          { -- slot controls }
          procedure AlChainScreen(  name: PChar );
          procedure AlSlotDownload(  id: LongInt );
          procedure AlSlotMail;
          procedure AlSlotPlugIn( name: PChar; cmd: PChar );
          procedure AlSlotPreview(  id: LongInt);
          procedure AlSlotLibrary(  id: LongInt );
          procedure AlSlotConference(  id: LongInt);
          procedure AlSlotBulletin(  name: PChar; title: PChar );

          { -- language independence routines }
          function  AlLoadLangDialog(  resourceID: PChar ):Bool;
          function  AlLoadLangLibrary:HWND;
          procedure AlFreeLangLibrary;
          function  AlLangLibrary:HWND;
          function  AlLangString(  id: Word; dest: PChar; len: Word ):PChar;

          { -- multi-user routines }
          function  AlCreateMulti(  count: Word; var dest: Pointer; size: Word ):Bool;
          function  AlCreateMultiAll(  size: Word ):Bool;
          function  AlCreateMultiSingle(  dest: LongInt; size: Word ):Bool;
          function  AlMultiLock:LPVOID;
          procedure AlMultiUnlock;
          function  AlMultiSelf:LongInt;

          { -- Direct COMM support (won't work with excalibur connections) }
          function  AlCommHandle:Integer;
          procedure AlSwitchDLL(  name: PChar; args: PChar );
          procedure AlRunDLL(  name: PChar; args: PChar );
          function  AlCarrier:Bool;
          function  AlSetNoHangup(  flag: Bool ):Bool;
 end;


 {TVersions.. host version checker.}
 type PFileVersionInfor = ^TFileVersionInfo;
     TFileVersionInfo = record
      Version: Array[0..250] of Char;
      Description: Array[0..250] of Char;
      Company: Array[0..250] of Char;
      InternalName: Array[0..250] of Char;
      Size: LongInt;
      dwFileVersionMS: LongInt;
      dwFileVersionLS: LongInt;
end;

type
  PLongInt = ^Longint;

type TVersions = class
     private
      VSize, VHandle: Longint;
      Buffer: PChar;
      Length, LangID, CharSetID: Word;
      TranslationInfo, Result: Pointer;
      StringFileInfo: Array[0..49] of Char;
      LangCharSetIDArray: Array[1..2] of Word;
      FVerInfo: TFileVersionInfo;
      FileName: Array[0..79] of Char;
      procedure GetInfo;
     public
      constructor Create( AFileName: TFileName );
      function GetVerInfo: TFileVersionInfo;
      function  DLLVerify(curOS: LongInt; var info: Pointer;
          var company: PChar; var intern: PChar; var ver: PChar;
          var descript: PChar ): Word;
end;

{ THostGraphicPlug }
type THostGraphicPlug = class( TPlugin )
      procedure BeforePlugInit; override;
end;

{THostClientPlug}
type THostClientPlug = class( TPlugin )
      procedure BeforePlugInit; override;
end;


function FileSizeEx( FileName: String ): LongInt;

 implementation

 procedure TPlugin.Die;
 begin
   AlSetState( PlugDie );
 end;

 procedure TPlugin.Zap;
 begin
   AlSetState( PlugZap );
 end;

 procedure TPlugin.BroadAllHere( var Data: Pointer );
  var DummyPtr: PByte;
      Ident: PByte;
      Counter: PInteger;
      tagUser: PtagAlBroadEnter;
      i: Integer;
      IncTrack: LongInt;  {Track pointer position so we can put it back...}
  begin
   IncTrack := 0;
   DummyPtr := data;
   Ident := PByte( DummyPtr );
   if (Ident^ = AL_BROAD_ALLHERE) and
      (FDoList) then
    begin
     {Get Count of users here}
     Inc(DummyPtr, SizeOf( Byte ));
     IncTrack := IncTrack + SizeOf(Byte);
     Counter := PInteger( DummyPtr );
     Inc( DummyPtr, SizeOf( Integer ));
     IncTrack := SizeOf(Byte)+SizeOf(Integer);

     {Loop through them and store to list. }
     for i := 0 to Counter^ - 1 do
      begin
        New(tagUser);
        System.Move(DummyPtr^, tagUser^, SizeOf(tagAlBroadEnter));
        FWhoIsHere.Add(tagUser);

       {Increment to next user here}
        Inc (DummyPtr, SizeOf( tagAlBroadEnter ));
        IncTrack := IncTrack + SizeOf(tagAlBroadEnter);
      end;
    end;
    Dec( PByte(DummyPtr), IncTrack );
  end;

  procedure TPlugin.BroadEnter( var Data: Pointer );
  var DummyPtr: PByte;
      Ident: PByte;
      Counter: PInteger;
      tagUser: PtagAlBroadEnter;
      IncTrack: LongInt;
  begin
   IncTrack := 0;
   DummyPtr := data;
   Ident := PByte( DummyPtr );
   if (Ident^ = AL_BROAD_ENTER) and
      (FDoList) then
    begin
     { Increment pointer past state variable}
      Inc(DummyPtr, SizeOf( Byte ));
      IncTrack := IncTrack + SizeOf(Byte);

     {If it isn't this user then add user.}
      if (PtagAlBroadEnter( DummyPtr )^.UserId <> AlGetUserId) then
       begin
        New(tagUser);
        System.Move(DummyPtr^, tagUser^, SizeOf(tagAlBroadEnter));
        FWhoIsHere.Add(tagUser);
       end;
   end;
   Dec(PByte(DummyPtr), IncTrack);
  end;

  procedure TPlugin.BroadLeave( var Data: Pointer );
  var ThisMultiId: LongInt;
      DummyPtr: PByte;
      Ident: PByte;
      i: Integer;
      IncTrack: LongInt;
  begin
   IncTrack := 0;
   DummyPtr := data;
   Ident := PByte( DummyPtr );

   if (Ident^ = AL_BROAD_LEAVE) and
      (FDoList) then
    begin
     {Get rid of state variable.}
      Inc( DummyPtr, SizeOf( Byte ) );
      IncTrack := IncTrack + SizeOf(Byte);

      ThisMultiId := LongInt( DummyPtr^ );

      {Find it and delete it..}
      for i := 0 to FWhoIsHere.Count-1 do
       if (LongInt(FWhoIsHere.Items[i]^) = ThisMultiId) then
          FWhoIsHere.Delete(i);

       Dec(PByte(DummyPtr), IncTrack);
    end;

  end;

  procedure TPlugin.SetSlot( Slot: LPSLOT );
  begin
   FSlot := Slot;
  end;

  procedure TPlugin.InitLanguage;
  begin
   { Have not tested this yet. Not yet sure about language stuff...}
  end;

  function  TPlugin.GetUserCount: Word;
  begin
    if (FDoList) and
       (FWhoIsHere <> nil ) then
     Result := FWhoIsHere.Count
    else
     Result := 0;
  end;

  function  TPlugin.GetUserFromList( Index: Integer ): tagAlBroadEnter;
  var AUser: tagAlBroadEnter;
  begin
   if (FDoList) and
      (FWhoIsHere <> nil ) then
   begin
    if (Index >= 0) and
       (Index < FWhoIsHere.Count) then
     begin
      System.Move( FWhoIsHere.Items[Index]^, AUser, SizeOf( tagAlBroadEnter ) );
      Result := AUser;
     end
    else MessageBox(0,'User index out of bounds in procedure GetUserFromList...',
                     'Index Out of Bounds.', mb_Ok or mb_IconHand );
   end;
  end;

  procedure TPlugin.SetOtherFiles( TheOtherFiles: TStrings );
  begin
    FOtherFiles.Assign(TheOtherFiles);
    FOtherFilesSent := 0;
  end;

  constructor TPlugin.Create( var Slot: LPSLOT );
  begin
   inherited Create;
   FMultiLingual := False;
   FWhoIsHere    := nil;
   FSlot         := Slot;
   FDllName      := '';
   FOtherFiles   := TStringList.Create;
   FDoList       := False;
   FWindowTitle  := '';
   FArguments    := '';
   FInitialized  := False;
 end;

  destructor TPlugin.Destroy;
  begin
    if FMultiLingual then AlFreeLangLibrary;
    if Assigned(FWhoIsHere) then FWhoIsHere.Free;
    if Assigned(FOtherFiles) then FOtherFiles.Free;
  end;

  procedure TPlugin.LoadOtherFilesFromFile( FileName: String );
  begin
   try
    FOtherFiles.LoadFromFile(FileName);
   except
    Abort;
   end;
  end;

  function TPlugin.SendBlock( var Block; Len: Word ): Boolean;
  var p: Pointer;
  begin
   Result := False;
   if Len > MaxBlockLength then Exit;
   GetMem( p, Len );
   try
    try
      System.Move( Block, p^, Len );
      AlAddBlock( p, Len );
      Result := True;
    finally
      FreeMem( p, Len );
    end;
   except
    Abort;
   end;
  end;

  function TPlugin.SendCommand( Command: Word ): Boolean;
  var Cmd: Word;
  begin
   Cmd := Command;
   AlAddBlock( @Cmd, SizeOf( Word ) );
  end;

  procedure TPlugin.BeforePlugInit;
  begin
   { Do nothing.}
  end;

  function TPlugin.PlgType( var Title: PChar ): Word;
  begin
   if (FPlugType = ptClientMulti) or
      (FPlugType = ptHiddenMulti) or
      (FPlugType = ptGraphicMulti ) then
     FMultiUser := True
   else
     FMultiUser := False;

    if FWindowTitle <> '' then
      StrPCopy(Title,FWindowTitle+#0)
    else
      StrPCopy(Title,FDllName+#0+FArguments+#0);

    if FDoList then
     begin
      FWhoIsHere := TList.Create;
      FWhoIsHere.Capacity := UserListCapacity;
     end;

    case FPlugType of
     ptGraphic     : Result := AL_WIN_GRAPHIC;
     ptGraphicMulti: Result := AL_WIN_GRAPHIC or AL_WIN_MULTI;
     ptHidden      : Result := AL_WIN_HIDDEN;
     ptHiddenMulti : Result := AL_WIN_HIDDEN or AL_WIN_MULTI;
     ptClient      : Result := AL_WIN_PLUGIN;
     ptClientMulti : Result := AL_WIN_PLUGIN or AL_WIN_MULTI;
    end;
  end;

  function TPlugin.Dialog( Return: Word ): Word;
  begin
    Result := AL_OK;
  end;

  function TPlugin.GrailData( var Data: Pointer; More: Bool ): Bool;
  begin
   Result := FALSE;
  end;

  function TPlugin.Mouse( Code: Byte; X,Y: Word ): Word;
  begin
   Result := AL_OK;
  end;

  function TPlugin.Button( Id: Word ): Word;
  begin
   Result := AL_OK;
  end;

  function TPlugin.DLLSize: LongInt;
  var TotalSize: LongInt;
      i: Integer;
  begin
   TotalSize := 0;
   TotalSize := FileSizeEx(FDllName);
   if (FOtherFiles.Count > 0) then
    for i := 0 to FOtherFiles.Count-1 do
      TotalSize := TotalSize + FileSizeEx(FOtherFiles[i]);
   Result := TotalSize;
  end;

  function TPlugin.Unique( args, cMessage: PChar): Bool;
  begin
   Result := not FMultiUser;
   StrPCopy( cMessage, 'Sorry, this is a single user Plug-In...');
  end;

 function TPlugin.NextDLL( var Name: PChar; NameLimit: Word;
                          var Dest: PChar; DestLimit: Word ):Bool;
 begin
   if FDllSent = False then
     begin
      StrPCopy( name, FDllName );
      StrPCopy( dest, FDllName );
      FDllSent := True;
      Result := True;
     end
   else if (FDllSent = True) and (FOtherFilesSent<FOtherFiles.Count) then
     begin
       StrPCopy( name, FOtherFiles[FOtherFilesSent] );
       StrPCopy( dest, FOtherFiles[FOtherFilesSent] );
       FOtherFilesSent := FOtherFilesSent + 1;
       Result := True;
     end
   else
     Result := False;

end;


  function TPlugin.CheckDLL( CurOS: LongInt;
                           ClientDllInfo: Pointer;
 		           var Company: PChar; var Intern: PChar;
                           var Ver: PChar; var Description: PChar ): Word;
  var VerObj: TVersions;
  begin
   VerObj := TVersions.Create(FDllName);
   try
    Result := VerObj.DllVerify(CurOS, ClientDllInfo,Company, Intern, Ver, Description );
    if Result = NODLL_OK then FDllSent := True;
   finally
    VerObj.Free;
   end;
  end;

  function TPlugin.Broadcast( var Data: Pointer; Len: Word ): Word;
  var Dummy: Pointer;
      Ident: PByte;
  begin
   Dummy := Data;
   Ident := PByte( Dummy );
   case Ident^ of
    AL_BROAD_ALLHERE: begin
                       BroadAllHere( Data );
                      end;

    AL_BROAD_ENTER:   begin
                       BroadEnter( Data );
                      end;

    AL_BROAD_LEAVE:   begin
                       BroadLeave( Data );
                      end;
   end;
   Result := AL_OK;
end;


 function TPlugin.Command( var Cmd: Pointer; Len: Word ): Word;
 var TheWord: Word;
 begin
   TheWord := PWord( Cmd )^;
   Result := AL_OK;

   case TheWord of

      PlugDie: begin
                SendCommand( PlugDie );
                AlSetState( PlugDie );
               end;

      PlugZap: begin
                 SendCommand( PlugZap );
                 AlSetState( PlugZap );
                end;

      PlugInit: begin
                 SendCommand( PlugInit );
                 Result := AL_OK;
                 FInitialized := True;
                end;

    end;
  if not FInitialized then Result := AL_IDLE;
 end;


 function TPlugin.Process: Word;
 var CurrentState: Word;
 begin
  CurrentState := AlGetState;
  Result := AL_OK;

  case CurrentState of

     PlugDie: Result := AL_DIE;
     PlugZap: Result := AL_HANGUP;

  end;

  if (FPlugType = ptClient) or
     (FPlugType = ptClientMulti) then
   begin
    if not FInitialized then
     begin
      Result := AL_IDLE;
      Exit;
     end;
   end
  else FInitialized := True;

 end;

  function TPlugin.Receive( var Data: Pointer; Len: Word;
                                         Sender: LongInt ): Word;       { return AL_OK; }
  begin
   Result := AL_OK;
  end;

  procedure TPlugin.Activate( Active: Boolean );
  begin
   { No response nessesary.}
  end;

  procedure TPlugin.Paint( var Dc: HDC; Erase: Bool; Bound: PRect );
  begin
   {No response nessesary.}
  end;

  procedure TPlugin.Resize( Width: Word; Height: Word );
  begin
   {No response nessesary.}
  end;

  { Shells for library functions }
  function TPlugin.AlSlot:Word;
  begin
    Result := Alchemw.AlSlot(FSlot);
  end;

  function TPlugin.AlArgs( dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlArgs( FSlot, dest, len );
  end;

  procedure TPlugin.AlCaption( title: PChar );
  begin
   Alchemw.AlCaption( FSlot, title );
  end;

  procedure TPlugin.AlLogWrite( text: PChar );
  begin
   Alchemw.AlLogWrite( FSlot, text );
  end;

  function TPlugin.AlAlloc( size: LongInt ):HWND;
  begin
    Result := Alchemw.AlAlloc( FSlot, size );
  end;

  procedure TPlugin.AlFree;
  begin
    Alchemw.AlFree( FSlot );
  end;

  function TPlugin.AlLock: LPVOID;
  begin
    Result := Alchemw.AlLock( FSlot );
  end;

  procedure TPlugin.AlUnlock;
  begin
    Alchemw.AlUnlock( FSlot );
  end;

  function TPlugin.AlReAlloc( size: LongInt ):HWND;
  begin
   Result := Alchemw.AlReAlloc( FSlot, size );
  end;

  function TPlugin.AlGetHandle:HWND;
  begin
   Result := Alchemw.AlGetHandle( FSlot );
  end;

  procedure TPlugin.AlSetHandle(dHandle: HWND );
  begin
    Alchemw.AlSetHandle( FSlot, dHandle );
  end;

  function TPlugin.AlGetPointer:LPVOID;
  begin
    Result := Alchemw.AlGetPointer( FSlot );
  end;

  procedure TPlugin.AlSetPointer( userPtr: LPVOID );
  begin
    Alchemw.AlSetPointer( FSlot, userPtr );
  end;

  procedure TPlugin.AlAddBlock( data: LPVOID; len: Word );
  begin
   Alchemw.AlAddBlock( FSlot, data, len );
  end;

  function TPlugin.AlMainWindow:HWND;
  begin
   Result := Alchemw.AlMainWindow( FSlot );
  end;

  procedure TPlugin.AlSetWindowOrg( hdc: HDC );
  begin
   Alchemw.AlSetWindowOrg( FSlot, hdc );
  end;

  function TPlugin.AlSetState( value: Word ):Word;
  begin
   Result := Alchemw.AlSetState( FSlot, value );
  end;

  function TPlugin.AlGetState:Word;
  begin
   Result := Alchemw.AlGetState( FSlot );
  end;


  procedure TPlugin.AlPushState( value: Word );
  begin
   Alchemw.AlPushState( FSlot, Value );
  end;

  function  TPlugin.AlPopState:Word;
  begin
   Result := Alchemw.AlPopState( FSlot );
  end;

  { -- screen handling routines }
  function  TPlugin.AlOpenScreen( name: PChar; ofs: OFSTRUCT; mode: Word ): Integer;
  begin
    Result := Alchemw.AlOpenScreen( FSlot, name, ofs, mode );
  end;

  function  TPlugin.AlOpenScratch(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
  begin
    Result := Alchemw.AlOpenScratch( FSlot, name, ofs, mode );
  end;

  function  TPlugin.AlOpenUserFile(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
  begin
    Result := Alchemw.AlOpenUserFile( FSlot, name, ofs, mode );
  end;

  function  TPlugin.AlOpenGlobalFile(  name: PChar; ofs: OFSTRUCT; mode: Word ):Integer;
  begin
    Result := Alchemw.AlOpenGlobalFile( FSlot, name, ofs, mode );
  end;

  function  TPlugin.AlDateTimeNow:LongInt;
  begin
    Result := Alchemw.AlDateTimeNow;
  end;

  procedure TPlugin.AlDateDecode( date: LongInt; var year, month, day: Word );
  begin
    Alchemw.AlDateDecode( date, year, month, day );
  end;

  function  TPlugin.AlDateEncode( year, month, day: Word ):LongInt;
  begin
   Result := Alchemw.AlDateEncode( year, month, day );
  end;

  procedure TPlugin.AlReleaseButton;
  begin
   Alchemw.AlReleaseButton( FSlot );
  end;

  procedure TPlugin.AlSound(  sound: PChar );
  begin
    Alchemw.AlSound( FSlot, sound );
  end;

  procedure TPlugin.AlMessageBox( TheMessage: PChar; title: PChar; code: Word );
  begin
    Alchemw.AlMessageBox( FSlot, TheMessage, title, code );
  end;

  procedure TPlugin.AlMouseDisable;
  begin
    Alchemw.AlMouseDisable( FSlot );
  end;

  procedure TPlugin.AlMouseEnable;
  begin
    Alchemw.AlMouseEnable( FSlot );
  end;

  procedure TPlugin.AlMouseRelease;
  begin
    Alchemw.AlMouseRelease( FSlot );
  end;

  { -- Dialog processing functions }
  procedure TPlugin.AlExecDialog;
  begin
    Alchemw.AlExecDialog( FSlot );
  end;

  function  TPlugin.AlLoadDialog( resourceID: PChar; module: HWND ):Bool;
  begin
    Result := Alchemw.AlLoadDialog( FSlot, resourceID, module );
  end;

  procedure TPlugin.AlNewDialog( caption: PChar; x, y, w, h: Word );
  begin
     Alchemw.AlNewDialog( FSlot, caption, x, y, w, h );
  end;

  procedure TPlugin.AlDialogPress( id: Word );
  begin
    Alchemw.AlDialogPress( FSlot, id );
  end;

  procedure TPlugin.AlSetString( id: Word; data: PChar );
  begin
    Alchemw.AlSetString( FSlot, id, data );
  end;

  procedure TPlugin.AlSetCheck( id: Word );
  begin
    Alchemw.AlSetCheck( FSlot, id );
  end;

  procedure TPlugin.AlLbAddString( id: Word; data: PChar );
  begin
   Alchemw.AlLbAddString( FSlot, id, data );
  end;

  procedure TPlugin.AlCbAddString( id: Word; data: PChar );
  begin
   Alchemw.AlCbAddString( FSlot, id, data );
  end;

  procedure TPlugin.AlLimitText( id, len: Word );
  begin
   Alchemw.AlLimitText( FSlot, id, len );
  end;

  procedure TPlugin.AlItemEdit( id, x, y, w, h: Word; style: LongInt );
  begin
   Alchemw.AlItemEdit( FSlot, id, x, y, w, h, style );
  end;

  procedure TPlugin.AlItemStatic(  id, x, y, w, h: Word; style: LongInt; text: PChar );
  begin
    Alchemw.AlItemStatic( FSlot, id, x, y, w, h, style, text );
  end;

  procedure TPlugin.AlItemButton(  id, x, y, w, h: Word; style: LongInt; text: PChar );
  begin
    Alchemw.AlItemButton( FSlot, id, x, y, w, h, style, text );
  end;

  procedure TPlugin.AlItemListBox(  id, x, y, w, h: Word; style: LongInt );
  begin
   Alchemw.AlItemListBox( FSlot, id, x, y, w, h, style );
  end;

  procedure TPlugin.AlItemComboBox(  id, x, y, w, h: Word; style: LongInt );
  begin
    Alchemw.AlItemComboBox( FSlot, id, x, y, w, h, style );
  end;

  procedure TPlugin.AlItemScrollBar(  id, x, y, w, h: Word; style: LongInt );
  begin
   Alchemw.AlItemScrollBar( FSlot, id, x, y, w, h, style );
  end;

  procedure TPlugin.AlItemBorShade(  id, x, y, w, h: Word; style: LongInt; text: PChar );
  begin
   Alchemw.AlItemBorShade( FSlot, id, x, y, w, h, style, text );
  end;

  procedure TPlugin.AlItemBorCheck(  id, x, y, w, h: Word; style: LongInt; text: PChar );
  begin
   Alchemw.AlItemBorCheck( FSlot, id, x, y, w, h, style, text );
  end;

  procedure TPlugin.AlItemBorRadio(  id, x, y, w, h: Word; style: LongInt; text: PChar );
  begin
    Alchemw.AlItemBorRadio( FSlot, id, x, y, w, h, style, text );
  end;

  procedure TPlugin.AlItemBorButton(  id, x, y, w, h: Word; style: LongInt );
  begin
   Alchemw.AlItemBorButton( FSlot, id, x, y, w, h, style );
  end;

  function  TPlugin.AlDialogReturn:Word;
  begin
   Result := Alchemw.AlDialogReturn( FSlot );
  end;

  function  TPlugin.AlGetText(  id: Word; dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetText( FSlot, id, dest, len );
  end;

  function  TPlugin.AlGetTextLen(  id: Word ):Integer;
  begin
    Result := Alchemw.AlGetTextLen( FSlot, id );
  end;

  function  TPlugin.AlGetCheck(  id: Word ):Bool;
  begin
    Result := Alchemw.AlGetCheck( FSlot, id );
  end;

  function  TPlugin.AlGetSelected(  id: Word ):Word;
  begin
   Result := Alchemw.AlGetSelected( FSlot, id );
  end;

  function  TPlugin.AlGetSelList(  id: Word; dest: Word; limit: Integer ):Word;
  begin
   Result := Alchemw.AlGetSelList( FSlot, id, dest, limit );
  end;

  function  TPlugin.AlGetSelCount(  id: Word ):Word;
  begin
    Result := Alchemw.AlGetSelCount( FSlot, id );
  end;

  function  TPlugin.AlGetScrollPos(  id: Word ):Word;
  begin
    Result := Alchemw.AlGetScrollPos( FSlot, id );
  end;

  procedure TPlugin.AlSetScrollPos(  id, pos: Word );
  begin
   Alchemw.AlSetScrollPos( FSlot, id, pos );
  end;

  procedure TPlugin.AlSetScrollRange(  id, lower, upper: Word );
  begin
    Alchemw.AlSetScrollRange( FSlot, id, lower, upper );
  end;

  procedure TPlugin.AlLbSelect(  id, pick: Word );
  begin
   Alchemw.AlLbSelect( FSlot, id, pick );
  end;

  procedure TPlugin.AlLbSelectItem(  id, pick: Word );
  begin
    Alchemw.AlLbSelectItem( FSlot, id, pick );
  end;

  procedure TPlugin.AlLbSelectRange(  id, lower, upper: Word );
  begin
   Alchemw.AlLbSelectRange( FSlot, id, lower, upper );
  end;

  procedure TPlugin.AlCbSelect(  id, pick: Word );
  begin
   Alchemw.AlCbSelect( FSlot, id, pick );
  end;

  { -- general Grail database access }
  function  TPlugin.AlGrailRequest(  table, fields, filter: PChar ):Bool;
  begin
   Result := Alchemw.AlGrailRequest( FSlot, table, fields, filter );
  end;

  function  TPlugin.AlGrailExecute(  command: PChar ):Bool;
  begin
   Result := Alchemw.AlGrailExecute( FSlot, command );
  end;

  function  TPlugin.AlCreateGrailInsert(  table: PChar; size: Word; needID: Bool ):Bool;
  begin
   Result := Alchemw.AlCreateGrailInsert( FSlot, table, size, needID );
  end;

  function  TPlugin.AlCreateGrailUpdate(  table: PChar; size: Word; id: LongInt ):Bool;
  begin
   Result := Alchemw.AlCreateGrailUpdate( FSlot, table, size, id );
  end;

  function  TPlugin.AlGrailLock:LPVOID;
  begin
   Result := Alchemw.AlGrailLock( FSlot );
  end;

  procedure TPlugin.AlGrailUnlock;
  begin
   Alchemw.AlGrailUnlock( FSlot );
  end;

  function  TPlugin.AlGrailLastInsert:LongInt;
  begin
   Result := Alchemw.AlGrailLastInsert( FSlot );
  end;


  { -- Current User Information }
  function  TPlugin.AlGetUserID:LongInt;
  begin
   Result := Alchemw.AlGetUserId( FSlot );
  end;

  function  TPlugin.AlGetUserULBytes:LongInt;
  begin
   Result := Alchemw.AlGetUserULBytes( FSlot );
  end;

  function  TPlugin.AlGetUserDLBytes:LongInt;
  begin
   Result := Alchemw.AlGetUserDLBytes( FSlot );
  end;

  function  TPlugin.AlGetUserBalance:LongInt;
  begin
   Result := Alchemw.AlGetUserBalance( FSlot );
  end;

  function  TPlugin.AlGetUserRevenue:LongInt;
  begin
   Result := Alchemw.AlGetUserRevenue( FSlot );
  end;

  function  TPlugin.AlGetUserLastCall:LongInt;
  begin
   Result := Alchemw.AlGetUserLastCall( FSlot );
  end;

  function  TPlugin.AlGetUserDayByteLeft:LongInt;
  begin
   Result := Alchemw.AlGetUserDayByteLeft( FSlot );
  end;

  function  TPlugin.AlGetUserWeekByteLeft:LongInt;
  begin
   Result := Alchemw.AlGetUserWeekByteLeft( FSlot );
  end;

  function  TPlugin.AlGetUserMonthByteLeft:LongInt;
  begin
    Result := Alchemw.AlGetUserMonthByteLeft( FSlot );
  end;

  function  TPlugin.AlGetUserPermByte:LongInt;
  begin
   Result := Alchemw.AlGetUserPermByte( FSlot );
  end;

  function  TPlugin.AlGetUserPermTime:LongInt;
  begin
   Result := Alchemw.AlGetUserPermTime( FSlot );
  end;

  function  TPlugin.AlGetUserFirstCall:LongInt;
  begin
   Result := Alchemw.AlGetUserFirstCall( FSlot );
  end;

  function  TPlugin.AlGetUserExpiration:LongInt;
  begin
   Result := Alchemw.AlGetUserExpiration( FSlot );
  end;

  function  TPlugin.AlGetUserCallByte:LongInt;
  begin
   Result := Alchemw.AlGetUserCallByte( FSlot );
  end;

  function  TPlugin.AlGetUserDayByte:LongInt;
  begin
   Result := Alchemw.AlGetUserDayByte( FSlot );
  end;

  function  TPlugin.AlGetUserWeekByte:LongInt;
  begin
   Result := Alchemw.AlGetUserWeekByte( FSlot );
  end;

  function  TPlugin.AlGetUserMonthByte:LongInt;
  begin
   Result := Alchemw.AlGetUserMonthByte( FSlot );
  end;

  function  TPlugin.AlGetUserBirth:LongInt;
  begin
   Result := Alchemw.AlGetUserBirth( FSlot );
  end;

  function  TPlugin.AlGetUserOtherMail:LongInt;
  begin
   Result := Alchemw.AlGetUserOtherMail( FSlot );
  end;

  function  TPlugin.AlGetUserULFiles:Word;
  begin
   Result := Alchemw.AlGetUserULFiles( FSlot );
  end;

  function  TPlugin.AlGetUserDLFiles:Word;
  begin
    Result := Alchemw.AlGetUserDLFiles( FSlot );
  end;

  function  TPlugin.AlGetUserTotalCall:Word;
  begin
    Result := Alchemw.AlGetUserTotalCall( FSlot );
  end;

  function  TPlugin.AlGetUserDayTimeLeft:Word;
  begin
    Result := Alchemw.AlGetUserDayTimeLeft( FSlot );
  end;

  function  TPlugin.AlGetUserWeekTimeLeft:Word;
  begin
   Result := Alchemw.AlGetUserWeekTimeLeft( FSlot );
  end;

  function  TPlugin.AlGetUserMonthTimeLeft:Word;
  begin
   Result := Alchemw.AlGetUserMonthTimeLeft( FSlot );
  end;

  function  TPlugin.AlGetUserByteRatio:Word;
  begin
    Result := Alchemw.AlGetUserByteRatio( FSlot );
  end;

  function  TPlugin.AlGetUserFileRatio:Word;
  begin
   Result := Alchemw.AlGetUserFileRatio( FSlot );
  end;

  function  TPlugin.AlGetUserCallTime:Word;
  begin
   Result := Alchemw.AlGetUserCallTime( FSlot );
  end;

  function  TPlugin.AlGetUserDayTime:Word;
  begin
    Result := Alchemw.AlGetUserDayTime( FSlot );
  end;

  function  TPlugin.AlGetUserWeekTime:Word;
  begin
   Result := Alchemw.AlGetUserWeekTime( FSlot );
  end;

  function  TPlugin.AlGetUserMonthTime:Word;
  begin
   Result := Alchemw.AlGetUserMonthTime( FSlot );
  end;

  function  TPlugin.AlGetUserLanguage:Word;
  begin
   Result := Alchemw.AlGetUserLanguage( FSlot );
  end;

  function  TPlugin.AlGetUserFirst(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserFirst( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserMiddle(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserMiddle( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserLast(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserLast( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserHandle(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetUserHandle( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPassword(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetUserPassword( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserTitle(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetUserTitle( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPosition(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserPosition( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserCompany(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserCompany( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserAddress1(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserAddress1( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserAddress2(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetUserAddress2( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserCity(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserCity( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserState(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserState( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserCountry(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserCountry( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPostal(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetUserPostal( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPhoneHome(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserPhoneHome( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPhoneWork(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserPhoneWork( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPhoneFax(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserPhoneFax( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserPhoneBBS(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserPhoneBBS( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserFirstVerify(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserFirstVerify( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserLastVerify(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserLastVerify( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserLocalNet(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetUserLocalNet( FSlot, dest, len );
  end;

  function  TPlugin.AlGetUserNetID(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetUserNetID( FSlot, dest, len );
  end;

  {function  AlGetUserComment(  dest: PChar; len: Word ):PChar;}
  function  TPlugin.AlGetUserFlag(  dest: PChar; index: Word ):Bool;
  begin
   Result :=  Alchemw.AlGetUserFlag( FSlot, dest, index );
  end;

  function  TPlugin.AlGetUserFlagCount:Word;
  begin
   Result := Alchemw.AlGetUserFlagCount( FSlot );
  end;

  function  TPlugin.AlIsUserFlag(  flag: PChar ):Bool;
  begin
    Result := Alchemw.AlIsUserFlag( FSlot, flag );
  end;

  procedure TPlugin.AlSetUserFlag(  name: PChar );
  begin
    Alchemw.AlSetUserFlag( FSlot, name );
  end;

  procedure TPlugin.AlClearUserFlag(  name: PChar );
  begin
   Alchemw.AlClearUserFlag( FSlot, name );
  end;

  procedure TPlugin.AlSetUserULBytes(  value: LongInt );
  begin
   Alchemw.AlSetUserULBytes( FSlot, value );
  end;

  procedure TPlugin.AlSetUserDLBytes(  value: LongInt );
  begin
   Alchemw.AlSetUserDLBytes( FSlot, value );
  end;

  procedure TPlugin.AlSetUserBalance(  value: LongInt );
  begin
   Alchemw.AlSetUserBalance( FSlot, value );
  end;

  procedure TPlugin.AlSetUserRevenue(  value: LongInt );
  begin
   Alchemw.AlSetUserRevenue( FSlot, value );
  end;

  procedure TPlugin.AlSetUserLastCall(  value: LongInt );
  begin
   Alchemw.AlSetUserLastCall( FSlot, Value );
  end;

  procedure TPlugin.AlSetUserDayByteLeft(  value: LongInt );
  begin
   Alchemw.AlSetUserDayByteLeft( FSlot, value );
  end;

  procedure TPlugin.AlSetUserWeekByteLeft(  value: LongInt );
  begin
   Alchemw.AlSetUserWeekByteLeft( FSlot, Value );
  end;

  procedure TPlugin.AlSetUserMonthByteLeft(  value: LongInt );
  begin
    Alchemw.AlSetUserMonthByteLeft( FSlot, value );
  end;

  procedure TPlugin.AlSetUserPermByte(  value: LongInt );
  begin
    Alchemw.AlSetUserPermByte( FSlot, Value );
  end;

  procedure TPlugin.AlSetUserPermTime(  value: LongInt );
  begin
   Alchemw.AlSetUserPermTime( FSlot, Value );
  end;

  procedure TPlugin.AlSetUserFirstCall(  value: LongInt );
  begin
   Alchemw.AlSetUserFirstCall( FSlot, value );
  end;

  procedure TPlugin.AlSetUserExpiration(  value: LongInt );
  begin
   Alchemw.AlSetUserExpiration( FSlot, Value );
  end;

  procedure TPlugin.AlSetUserCallByte(  value: LongInt );
  begin
   Alchemw.AlSetUserCallByte( FSlot, value );
  end;

  procedure TPlugin.AlSetUserDayByte(  value: LongInt );
  begin
   Alchemw.AlSetUserDayByte( FSlot, value );
  end;

  procedure TPlugin.AlSetUserWeekByte(  value: LongInt );
  begin
   Alchemw.AlSetUserWeekByte( FSlot, value );
  end;

  procedure TPlugin.AlSetUserMonthByte(  value: LongInt );
  begin
   Alchemw.AlSetUserMonthByte( FSlot, value );
  end;

  procedure TPlugin.AlSetUserBirth(  value: LongInt );
  begin
   Alchemw.AlSetUserBirth( FSlot, value );
  end;

  procedure TPlugin.AlSetUserOtherMail(  value: LongInt );
  begin
    Alchemw.AlSetUserOtherMail( FSlot, value );
  end;

  procedure TPlugin.AlSetUserULFiles(  value: Word );
  begin
    Alchemw.AlSetUserULFiles( FSlot, value );
  end;

  procedure TPlugin.AlSetUserDLFiles(  value: Word );
  begin
   Alchemw.AlSetUserDLFiles( FSlot, value );
  end;

  procedure TPlugin.AlSetUserTotalCall(  value: Word );
  begin
   Alchemw.AlSetUserTotalCall( FSlot, value );
  end;

  procedure TPlugin.AlSetUserDayTimeLeft(  value: Word );
  begin
   Alchemw.AlSetUserDayTimeLeft( FSlot, value );
  end;

  procedure TPlugin.AlSetUserWeekTimeLeft(  value: Word );
  begin
   Alchemw.AlSetUserWeekTimeLeft( FSlot, value );
  end;

  procedure TPlugin.AlSetUserMonthTimeLeft(  value: Word );
  begin
   Alchemw.AlSetUserMonthTimeLeft( FSlot, value );
  end;

  procedure TPlugin.AlSetUserByteRatio(  value: Word );
  begin
   Alchemw.AlSetUserByteRatio( FSlot, value );
  end;

  procedure TPlugin.AlSetUserFileRatio(  value: Word );
  begin
    Alchemw.AlSetUserFileRatio( FSlot, value );
  end;

  procedure TPlugin.AlSetUserCallTime(  value: Word );
  begin
   Alchemw.AlSetUserCallTime( FSlot, value );
  end;

  procedure TPlugin.AlSetUserDayTime(  value: Word );
  begin
   Alchemw.AlSetUserDayTime( FSlot, value );
  end;

  procedure TPlugin.AlSetUserWeekTime(  value: Word );
  begin
   Alchemw.AlSetUserWeekTime( FSlot, value );
  end;

  procedure TPlugin.AlSetUserMonthTime(  value: Word );
  begin
   Alchemw.AlSetUserMonthTime( FSlot, value );
  end;

  procedure TPlugin.AlSetUserLanguage(  value: Word );
  begin
    Alchemw.AlSetUserLanguage( FSlot, Value );
  end;

  function  TPlugin.AlSetUserFirst(  data: PChar ):PChar;
  begin
    Result :=  Alchemw.AlSetUserFirst( FSlot, data );
  end;

  function  TPlugin.AlSetUserMiddle(  data: PChar ):PChar;
  begin
   Result :=  Alchemw.AlSetUserMiddle( FSlot, data );
  end;

  function  TPlugin.AlSetUserLast(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserLast( FSlot, data );
  end;

  function  TPlugin.AlSetUserHandle(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserHandle( FSlot, data );
  end;

  function  TPlugin.AlSetUserPassword(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPassword( FSlot, data );
  end;

  function  TPlugin.AlSetUserTitle(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserTitle( FSlot, data );
  end;

  function  TPlugin.AlSetUserPosition(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPosition( FSlot, Data );
  end;

  function  TPlugin.AlSetUserCompany(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserCompany( FSlot, data );
  end;

  function  TPlugin.AlSetUserAddress1(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserAddress1( FSlot, data );
  end;

  function  TPlugin.AlSetUserAddress2(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserAddress2( FSlot, data );
  end;

  function  TPlugin.AlSetUserCity(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserCity( FSlot, data );
  end;

  function  TPlugin.AlSetUserState(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserState( FSlot, data );
  end;

  function  TPlugin.AlSetUserCountry(  data: PChar ):PChar;
  begin
   Result :=  Alchemw.AlSetUserCountry( FSlot, data );
  end;

  function  TPlugin.AlSetUserPostal(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPostal( FSlot, data );
  end;

  function  TPlugin.AlSetUserPhoneHome(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPhoneHome( FSlot, data );
  end;

  function  TPlugin.AlSetUserPhoneWork(  data: PChar ):PChar;
  begin
   Result :=  Alchemw.AlSetUserPhoneWork( FSlot, data );
  end;

  function  TPlugin.AlSetUserPhoneFax(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPhoneFax( FSlot, data );
  end;

  function  TPlugin.AlSetUserPhoneBBS(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserPhoneBBS( FSlot, data );
  end;

  function  TPlugin.AlSetUserFirstVerify(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserFirstVerify( FSlot, data );
  end;

  function  TPlugin.AlSetUserLastVerify(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserLastVerify( FSlot, data );
  end;

  function  TPlugin.AlSetUserLocalNet(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserLocalNet( FSlot, data );
  end;

  function  TPlugin.AlSetUserNetID(  data: PChar ):PChar;
  begin
   Result := Alchemw.AlSetUserNetID( FSlot, data );
  end;

  {function  AlSetUserComment(  data: PChar ):PChar;}

  { -- Current Port Information }
  function  TPlugin.AlGetPortID:LongInt;
  begin
   Result := Alchemw.AlGetPortID( FSlot );
  end;

  function  TPlugin.AlGetPortLastUser:LongInt;
  begin
   Result := Alchemw.AlGetPortLastUser( FSlot );
  end;

  function  TPlugin.AlGetPortCalls:LongInt;
  begin
   Result := Alchemw.AlGetPortCalls( FSlot );
  end;

  function  TPlugin.AlGetPortBaud:LongInt;
  begin
   Result := Alchemw.AlGetPortBaud( FSlot );
  end;

  function  TPlugin.AlGetPortTemplate:LongInt;
  begin
   Result := Alchemw.AlGetPortTemplate( FSlot );
  end;

  function  TPlugin.AlGetPortGuest:LongInt;
  begin
   Result := Alchemw.AlGetPortGuest( FSlot );
  end;

  function  TPlugin.AlGetPortWinStyle:Word;
  begin
   Result := Alchemw.AlGetPortWinStyle( FSlot );
  end;

  function  TPlugin.AlGetPortWinX:Word;
  begin
    Result := Alchemw.AlGetPortWinX( FSlot );
  end;

  function  TPlugin.AlGetPortWinY:Word;
  begin
    Result :=  Alchemw.AlGetPortWinY( FSlot );
  end;

  function  TPlugin.AlGetPortWinW:Word;
  begin
   Result :=  Alchemw.AlGetPortWinW( FSlot );
  end;

  function  TPlugin.AlGetPortWinH:Word;
  begin
   Result := Alchemw.AlGetPortWinH( FSlot );
  end;

  function  TPlugin.AlGetPortBeepRepeat:Word;
  begin
   Result := Alchemw.AlGetPortBeepRepeat( FSlot );
  end;

  function  TPlugin.AlGetPortBeepInterval:Word;
  begin
    Result := Alchemw.AlGetPortBeepInterval( FSlot );
  end;

  function  TPlugin.AlGetPortPort:BYTE;
  begin
    Result := Alchemw.AlGetPortPort( FSlot );
  end;

  function  TPlugin.AlGetPortRingDetect:Bool;
  begin
   Result := Alchemw.AlGetPortRingDetect( FSlot );
  end;

  function  TPlugin.AlGetPortUseHangup:Bool;
  begin
   Result := Alchemw.AlGetPortUseHangup( FSlot );
  end;

  function  TPlugin.AlGetPortRingCount:BYTE;
  begin
    Result := Alchemw.AlGetPortRingCount( FSlot );
  end;

  function  TPlugin.AlGetPortTimeout:BYTE;
  begin
   Result := Alchemw.AlGetPortTimeout( FSlot );
  end;

  function  TPlugin.AlGetPortLockRate:Bool;
  begin
   Result := Alchemw.AlGetPortLockRate( FSlot );
  end;

  procedure TPlugin.AlGetPortDriveMap(  dest: PChar );
  begin
   Alchemw.AlGetPortDriveMap( FSlot, dest );
  end;

  function  TPlugin.AlGetPortNetName(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortNetName( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortSendExcal(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortSendExcal( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortLogin(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortLogin( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortNewUser(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetPortNewUser( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortModemName(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortModemName( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortReset(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortReset( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortInit(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortInit( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortOffHook(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortOffHook( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortOnHook(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortOnHook( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortAnswer(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortAnswer( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortRing(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortRing( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortCIDPrefix(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetPortCIDPrefix( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortCIDPrivate(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortCIDPrivate( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortCIDNone(  dest: PChar; len: Word ):PChar;
  begin
   Result :=  Alchemw.AlGetPortCIDNone( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortHangup(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortHangup( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortConnect( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect1200(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortConnect1200( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect2400(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortConnect2400( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect4800(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortConnect4800( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect9600(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortConnect9600( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect19k(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortConnect19k( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect38k(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetPortConnect38k( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortConnect57k(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortConnect57k( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortErrorCorrect(  index: Word; dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortErrorCorrect( FSlot, index, dest, len );
  end;

  function  TPlugin.AlGetPortEffBaud(  index: Word; dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortEffBaud( FSlot, index, dest, len );
  end;

  function  TPlugin.AlGetPortWaitMessage(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortWaitMessage( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortScratch(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortScratch( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortSoundName(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortSoundName( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortFirst(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortFirst( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortMiddle(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortMiddle( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortLast(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortLast( FSlot, dest, len );
  end;

  function  TPlugin.AlGetPortHandle(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetPortHandle( FSlot, dest, len );
  end;


  { -- system settings }
  function  TPlugin.AlGetSysMajorRev:Word;
  begin
   Result :=  Alchemw.AlGetSysMajorRev( FSlot );
  end;

  function  TPlugin.AlGetSysMinorRev:Word;
  begin
    Result := Alchemw.AlGetSysMinorRev( FSlot );
  end;

  function  TPlugin.AlGetSysPageFrom:Word;
  begin
   Result :=  Alchemw.AlGetSysPageFrom( FSlot );
  end;

  function  TPlugin.AlGetSysPageTo:Word;
  begin
   Result := Alchemw.AlGetSysPageTo( FSlot );
  end;

  function  TPlugin.AlGetSysByteCreditPct:Word;
  begin
   Result := Alchemw.AlGetSysByteCreditPct( FSlot );
  end;

  function  TPlugin.AlGetSysDayCarryPct:Word;
  begin
   Result := Alchemw.AlGetSysDayCarryPct( FSlot );
  end;

  function  TPlugin.AlGetSysWeekCarryPct:Word;
  begin
   Result := Alchemw.AlGetSysWeekCarryPct( FSlot );
  end;

  function  TPlugin.AlGetSysMonthCarryPct:Word;
  begin
   Result := Alchemw.AlGetSysMonthCarryPct( FSlot );
  end;

  function  TPlugin.AlGetSysDayCarryLimit:LongInt;
  begin
   Result := Alchemw.AlGetSysDayCarryLimit( FSlot );
  end;

  function  TPlugin.AlGetSysWeekCarryLimit:LongInt;
  begin
   Result := Alchemw.AlGetSysWeekCarryLimit( FSlot );
  end;

  function  TPlugin.AlGetSysMonthCarryLimit:LongInt;
  begin
   Result := Alchemw.AlGetSysMonthCarryLimit( FSlot );
  end;

  function  TPlugin.AlGetSysCanPage:Bool;
  begin
   Result := Alchemw.AlGetSysCanPage( FSlot );
  end;

  function  TPlugin.AlGetSysAdultAge:BYTE;
  begin
   Result := Alchemw.AlGetSysAdultAge( FSlot );
  end;

  function  TPlugin.AlGetSysLogClear:Bool;
  begin
   Result := Alchemw.AlGetSysLogClear( FSlot );
  end;

  function  TPlugin.AlGetSysPageClear:Bool;
  begin
   Result := Alchemw.AlGetSysPageClear( FSlot );
  end;

  function  TPlugin.AlGetSysMinimumTime:BYTE;
  begin
   Result := Alchemw.AlGetSysMinimumTime( FSlot );
  end;

  function  TPlugin.AlGetSysChargePartial:Bool;
  begin
    Result := Alchemw.AlGetSysChargePartial( FSlot );
  end;

  function  TPlugin.AlGetSysNoULClock:Bool;
  begin
   Result := Alchemw.AlGetSysNoULClock( FSlot );
  end;

  function  TPlugin.AlGetSysCreditNow:Bool;
  begin
   Result := Alchemw.AlGetSysCreditNow( FSlot );
  end;

  function  TPlugin.AlGetSysSystemName(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysSystemName( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysSerial(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysSerial( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysCreditCard(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysCreditCard( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysDefTopic(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysDefTopic( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysTimeZone(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetSysTimeZone( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysChat(  index: Word; dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysChat( FSlot, index, dest, len );
  end;

  function  TPlugin.AlGetSysScreenDir(  dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlGetSysScreenDir( FSlot, dest, len );
  end;

  procedure TPlugin.AlGetSysLogFlags(  dest: PChar );
  begin
   Alchemw.AlGetSysLogFlags( FSlot, dest );
  end;

  function  TPlugin.AlGetSysUserDir(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetSysUserDir( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysPreviewDir(  dest: PChar; len: Word ):PChar;
  begin
    Result := Alchemw.AlGetSysPreviewDir( FSlot, dest, len );
  end;

  function  TPlugin.AlGetSysNoAttachments:Bool;
  begin
   Result := Alchemw.AlGetSysNoAttachments( FSlot );
  end;

  function  TPlugin.AlGetSysPreviewCharge:Word;
  begin
    Result := Alchemw.AlGetSysPreviewCharge( FSlot );
  end;

  function  TPlugin.AlGetSysHandles:Bool;
  begin
   Result := Alchemw.AlGetSysHandles( FSlot );
  end;

  { -- Graphic commands }
  procedure TPlugin.AlScreenStart(  color: BYTE );
  begin
    Alchemw.AlScreenStart( FSlot, color );
  end;

  procedure TPlugin.AlScreenEnd;
  begin
    Alchemw.AlScreenEnd( FSlot );
  end;

  procedure TPlugin.AlLine(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
  begin
   Alchemw.AlLine( FSlot,x1, y1, x2, y2, color, mode, width );
  end;

  procedure TPlugin.AlRectangle(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
  begin
   Alchemw.AlRectangle( FSlot,x1, y1, x2, y2, color, mode, width );
  end;

  procedure TPlugin.AlEllipse(  x1, y1, x2, y2: Word; color, mode: BYTE; width: Word );
  begin
   Alchemw.AlEllipse( FSlot,x1, y1, x2, y2, color, mode, width );
  end;

  procedure TPlugin.AlFRectangle(  x1, y1, x2, y2: Word; color, mode: BYTE;
                                  width: Word; fg, bg, fill: BYTE; transparent: Bool );
  begin
   Alchemw.AlFRectangle( FSlot,x1,y1,x2,y2,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlFEllipse(  x1, y1, x2, y2: Word; color, mode: BYTE;
                                width: Word; fg, bg, fill: BYTE; transparent: Bool );
  begin
   Alchemw.AlFEllipse( FSlot,x1,y1,x2,y2,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlRoundRect(  x1, y1, x2, y2: Word; xe, ye: Word;
                                 color, mode: BYTE; width: Word );
  begin
   Alchemw.AlRoundRect( FSlot,x1,y1,x2,y2,xe,ye,color,mode,width);
  end;

  procedure TPlugin.AlFRoundRect(  x1, y1, x2, y2: Word; xe, ye: Word;
                                  color, mode: BYTE; width: Word; fg, bg, fill: BYTE;
                                                                  transparent: Bool );
  begin
   Alchemw.AlFRoundRect( FSlot,x1,y1,x2,y2,xe,ye,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlArc(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                                                  color, mode: BYTE; width: Word );
  begin
   Alchemw.AlArc( FSlot,x1,y1,x2,y2,x3,y3,x4,y4,color,mode,width);
  end;

  procedure TPlugin.AlPie(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                                                              color, mode: BYTE;
                                    width: Word; fg, bg, fill: BYTE; transparent: Bool );
  begin
    Alchemw.AlPie(FSlot,x1,y1,x2,y2,x3,y3,x4,y4,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlChord(  x1, y1, x2, y2, x3, y3, x4, y4: Word;
                             color, mode: BYTE; width: Word; fg, bg, fill: BYTE;
                             transparent: Bool );
  begin
    Alchemw.AlChord(FSlot,x1,y1,x2,y2,x3,y3,x4,y4,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlHit(  x1, y1, x2, y2: Word; id, key: BYTE  );
  begin
    Alchemw.AlHit(FSlot,x1,y1,x2,y2,id,key);
  end;

  procedure TPlugin.AlButton(  x1, y1, x2, y2: Word;
                               id, key, font, width, height, style,
                              color: BYTE; caption: PChar );
  begin
   Alchemw.AlButton(FSlot,x1,y1,x2,y2,id,key,font,width,height,style,color,caption);
  end;

  procedure TPlugin.AlPolygon(  count: Word; var point: Word;
                               color, mode: BYTE; width: Word );
  begin
   Alchemw.AlPolygon(FSlot,count,point,color,mode,width);
  end;

  procedure TPlugin.AlFPolygon(  count: Word; var point: Word;
                                color, mode: BYTE; width: Word;
                                fg, bg, fill: BYTE; transparent: Bool );
  begin
    Alchemw.AlFPolygon(FSlot,count,point,color,mode,width,fg,bg,fill,transparent);
  end;

  procedure TPlugin.AlText(  x1, y1, x2, y2: Word;
                            font, width, height, style, position,
                            color: BYTE; text: PChar );
  begin
   Alchemw.AlText(FSlot,x1,y1,x2,y2,font,width,height,style,position,color,text);
  end;

  procedure TPlugin.AlCard(  card: BYTE; x1, y1, x2, y2: Word );
  begin
   Alchemw.AlCard(FSlot,card,x1,y1,x2,y2);
  end;

  { -- International conversions }
  function  TPlugin.AlUserMoneyFormat(  dest: PChar; amt: LongInt ):PChar;
  begin
   Result := Alchemw.AlUserMoneyFormat( FSlot,dest,amt);
  end;

  function  TPlugin.AlSysMoneyFormat(  dest: PChar; amt: LongInt ):PChar;
  begin
   Result := Alchemw.AlSysMoneyFormat( FSlot,dest,amt );
  end;

  function  TPlugin.AlSysShortDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
  begin
   Result := Alchemw.AlSysShortDate( FSlot,date,dest,len );
  end;

  function  TPlugin.AlSysLongDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
  begin
   Result := Alchemw.AlSysLongDate(FSlot,date,dest,len);
  end;

  function  TPlugin.AlUserShortDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
  begin
   Result := Alchemw.AlUserShortDate(FSlot,date,dest,len);
  end;

  function  TPlugin.AlUserLongDate(  date: LongInt; dest: PChar; len: Integer ):PChar;
  begin
    Result := Alchemw.AlUserLongDate(FSlot,date,dest,len);
  end;

  function  TPlugin.AlSysTime(  date: LongInt; dest: PChar; len: Integer ): PChar;
  begin
   Result := Alchemw.AlSysTime(FSlot,date,dest,len);
  end;


  { -- slot controls }
  procedure TPlugin.AlChainScreen(  name: PChar );
  begin
    Alchemw.AlChainScreen(FSlot,Name);
  end;

  procedure TPlugin.AlSlotDownload(  id: LongInt );
  begin
   Alchemw.AlSlotDownload(FSlot,id);
  end;

  procedure TPlugin.AlSlotMail;
  begin
   Alchemw.AlSlotMail(FSlot);
  end;

  procedure TPlugin.AlSlotPlugIn( name: PChar; cmd: PChar );
  begin
    Alchemw.AlSlotPlugin(FSlot,name,cmd);
  end;

  procedure TPlugin.AlSlotPreview(  id: LongInt);
  begin
   Alchemw.AlSlotPreview(FSlot,id);
  end;

  procedure TPlugin.AlSlotLibrary(  id: LongInt );
  begin
   Alchemw.AlSlotLibrary(FSlot,id);
  end;

  procedure TPlugin.AlSlotConference(  id: LongInt);
  begin
   Alchemw.AlSlotConference(FSlot,id);
  end;

  procedure TPlugin.AlSlotBulletin(  name: PChar; title: PChar );
  begin
   Alchemw.AlSlotBulletin(FSlot,name,title);
  end;


  { -- language independence routines }
  function  TPlugin.AlLoadLangDialog(  resourceID: PChar ):Bool;
  begin
   Result := Alchemw.AlLoadLangDialog(FSlot,resourceID);
  end;

  function  TPlugin.AlLoadLangLibrary:HWND;
  begin
   Result := Alchemw.AlLoadLangLibrary(FSlot);
  end;

  procedure TPlugin.AlFreeLangLibrary;
  begin
    Alchemw.AlFreeLangLibrary(FSlot);
  end;

  function  TPlugin.AlLangLibrary:HWND;
  begin
   Result := Alchemw.AlLangLibrary(FSlot);
  end;

  function  TPlugin.AlLangString(  id: Word; dest: PChar; len: Word ):PChar;
  begin
   Result := Alchemw.AlLangString(FSlot,id,dest,len);
  end;


  { -- multi-user routines }
  function  TPlugin.AlCreateMulti(  count: Word; var dest: Pointer; size: Word ):Bool;
  begin
   Result := Alchemw.AlCreateMulti(FSlot,count,dest,size);
  end;

  function  TPlugin.AlCreateMultiAll(  size: Word ):Bool;
  begin
   Result := Alchemw.AlCreateMultiAll(FSlot,size);
  end;

  function  TPlugin.AlCreateMultiSingle(  dest: LongInt; size: Word ):Bool;
  begin
   Result := Alchemw.AlCreateMultiSingle( FSlot,dest,size );
  end;

  function  TPlugin.AlMultiLock:LPVOID;
  begin
   Result := Alchemw.AlMultiLock(FSlot);
  end;

  procedure TPlugin.AlMultiUnlock;
  begin
     Alchemw.AlMultiUnlock(FSlot);
  end;

  function  TPlugin.AlMultiSelf:LongInt;
  begin
   Result := Alchemw.AlMultiSelf(FSlot);
  end;

  { -- Direct COMM support (won't work with excalibur connections) }
  function  TPlugin.AlCommHandle:Integer;
  begin
   Result := Alchemw.AlCommHandle(FSlot);
  end;

  procedure TPlugin.AlSwitchDLL(  name: PChar; args: PChar );
  begin
    Alchemw.AlSwitchDLL(FSlot,name,args);
  end;

  procedure TPlugin.AlRunDLL(  name: PChar; args: PChar );
  begin
    Alchemw.AlRunDLL(FSlot,name,args);
  end;

  function  TPlugin.AlCarrier:Bool;
  begin
    Result :=  Alchemw.AlCarrier(FSlot);
  end;

  function  TPlugin.AlSetNoHangup(  flag: Bool ):Bool;
  begin
    Result := Alchemw.AlSetNoHangup( FSlot,flag );
  end;


{ Version Checker. }
constructor TVersions.Create( AFileName: TFileName );
 begin
  inherited Create;
  if not FileExists( AFileName ) then
   begin
    MessageBox(0, 'The Dll refered to dosn''t exist!', 'Problem', mb_ok);
   end
  else
   begin
    StrPCopy(FileName, AFileName);
    GetInfo;
   end;
 end;



procedure TVersions.GetInfo;
 var VersionInts: Pointer;
     Len: Word;
 begin
  { Get size of version info }
  VSize := GetFileVersionInfoSize(FileName, VHandle);

  { Allocate version info buffer }
  GetMem(Buffer, VSize + 1);

  { START GET VERSION INFO... }
 try
  if GetFileVersionInfo(FileName, VHandle, VSize, Buffer) then
    { Get translation info for Language / CharSet IDs }


    if VerQueryValue(Buffer, '\VarFileInfo\Translation',
                                       TranslationInfo, Length) then
     begin
      LangCharSetIDArray[1] := LoWord(PLongint(TranslationInfo)^);
      LangCharSetIDArray[2] := HiWord(PLongint(TranslationInfo)^);

      { Get company name }
      wvsPrintf(StringFileInfo,
                           '\StringFileInfo\%04x%04x\CompanyName',
                          LangCharSetIDArray);
      VerQueryValue(Buffer, StringFileInfo, Result, Length);
       StrCopy(FVerInfo.Company, PChar(Result) ) ;

      { Get file description }
      wvsPrintf(StringFileInfo,
                          '\StringFileInfo\%04x%04x\FileDescription',
                          LangCharSetIDArray);
      VerQueryValue(Buffer, StringFileInfo, Result, Length);
       StrCopy(FVerInfo.Description , PChar(Result)) ;

      { Get file version }
      wvsPrintf(StringFileInfo,
                          '\StringFileInfo\%04x%04x\FileVersion',
                          LangCharSetIDArray);
      VerQueryValue(Buffer, StringFileInfo, Result, Length);
       StrCopy(FVerInfo.Version, PChar(Result));

      { Get internal name}
      wvsPrintf(StringFileInfo,
                          '\StringFileInfo\%04x%04x\InternalName',
                          LangCharSetIDArray);
      VerQueryValue(Buffer, StringFileInfo, Result, Length);
       StrCopy(FVerInfo.InternalName, PChar(Result));

     GetMem( VersionInts, SizeOf( Tvs_FixedFileInfo ) + 100);
      try
       VerQueryValue(Buffer,'\', VersionInts,Len);
        FVerInfo.dwFileVersionMS := Pvs_FixedFileInfo(VersionInts)^.dwFileVersionMS;
        FVerInfo.dwFileVersionLS := Pvs_FixedFileInfo(VersionInts)^.dwFileVersionLS;
      finally
       FreeMem( VersionInts, SizeOf( Tvs_FixedFileInfo ) + 100 );
      end;

   end;
 finally
  FreeMem(Buffer, VSize + 1);
 end;
end;


function TVersions.GetVerInfo: TFileVersionInfo;
 begin
   Result := FVerInfo;
 end;


function  TVersions.DLLVerify(curOS: LongInt; var info: Pointer;
          var company: PChar; var intern: PChar; var ver: PChar;
          var descript: PChar ): Word;
const IsEmpty: PChar = '';
var VerInfo: TFileVersionInfo;
    AVerInfo: TFileVersionInfo;
    UserMS: LongInt;
    SysMS: LongInt;
    UserLS: LongInt;
    SysLS: LongInt;
    pchTrans: PChar;
begin
 {Zero out locals...}
 with VerInfo do
  begin
   FillChar(Version, 250, #0);
   FillChar(Description, 250, #0);
   FillChar(Company, 250, #0);
   FillChar(InternalName, 250, #0);
   Size := 0;
   dwFileVersionMS := 0;
   dwFileVersionLS := 0;
 end;

 VerInfo := GetVerInfo;

 Result := NODLL_OK;

 if (curOS <> VOS__WINDOWS16) then
    Result := NODLL_OVERWRITE
 else if ( lstrcmpi( VerInfo.Company, company ) <> 0 ) then
    Result := NODLL_OVERWRITE
 else if ( (Pvs_FixedFileInfo(info)^.dwFileVersionMS) < VerInfo.dwFileVersionMS ) then
    Result := NODLL_MAJOR
 else if ( (Pvs_FixedFileInfo(info)^.dwFileVersionMS shr 16) >
           (Pvs_FixedFileInfo(info)^.dwFileVersionMS shr 16) ) then
       begin
        UserMS := (Pvs_FixedFileInfo(info)^.dwFileVersionMS shr 16);
	SysMS :=  (VerInfo.dwFileVersionMS shr 16 );
	if ( UserMS > SysMS ) then
	 Result := NODLL_NEWER;
       end
 else if ( Pvs_FixedFileInfo(info)^.dwFileVersionLS < VerInfo.dwFileVersionLS ) then
          Result := NODLL_MINOR;

 if (StrComp( IsEmpty, Company ) = 0 ) or
    (StrComp( IsEmpty, Intern ) = 0 ) or
    (StrComp( IsEmpty, Ver ) = 0 ) or
    (StrComp( IsEmpty, Descript ) = 0 ) then
  Result := NODLL_NONE;

  StrCopy( Company, VerInfo.Company );
  StrCopy( Intern, VerInfo.InternalName );
  StrCopy( Descript, VerInfo.Description );
  StrCopy( Ver, VerInfo.Version );
end;

{THostGraphicPlug}
procedure THostGraphicPlug.BeforePlugInit;
begin
  inherited BeforePlugInit;
  BuildUserList := False;
  ClientDllName := '';
  MultiLingual  := False;
  ClientArgs    := '';
  WindowTitle   := 'THostGraphicPlug Object, Copyright 1996 Argon D. Helm';
  PlugType      := ptGraphicMulti;
end;


{THostClientPlug}
procedure THostClientPlug.BeforePlugInit;
const CstrArgs: String = 'THostClientPlug Sample Plugin.' + #13#10 +
                         'Copyright 1996 Argon D. Helm.';
begin
  inherited BeforePlugInit;
  BuildUserList := False;
  ClientDllName := 'DOPLCLNT.DLL';
  MultiLingual  := False;
  ClientArgs    := CstrArgs;
  WindowTitle   := '';
  PlugType      := ptClientMulti;
end;



{ File system save filesize procedure.}
function FileSizeEx( FileName: String ): LongInt;
var temp: Integer;
    size: Longint;
    ofs: TOfStruct;
    chrName: Array[0..250] of Char;
begin
 StrPCopy( chrName, FileName );
 temp := OpenFile( chrName, ofs, OF_READ );
	size := 0;
	if ( temp >= 0 ) then
         begin
	  size := _llseek( temp, 0, 2 );
	  _lclose( temp );
         end;
 Result := size;
end;

end.
