unit Cserve;

interface

uses WinTypes,
     WinProcs,
     Classes,
     SysUtils,
     Dopl,
     FileToss,
     Messages;

{ Flag to catch and process a file block.}
const
      CClientOutFileBlocks = 600;
      CHostInFileBlocks = 601;
      CRegisterFile = 602;
      CCancelFile = 603;
      CSendFile = 604;
      CInvalidFileName = 605;
      CGetFile = 606;
      CHostOutFileBlocks = 607;
      CClientInFileBlocks = 608;

      {File transfer cancellation. }
      CFileIsCanceled = 1;
      CFileNotFound = 2;


{TCustomServer}
type TCustomServer = class( THostClientPlug )
      private
       FFileList: TStringList;
       FOutFileStream: TTossFile;
       FInFileStream: TTossFile;
       FSendFileReady: Boolean;
       FCurrentSend: String;
       FDownLoadDir: String;
     public
       function Command( var Cmd: Pointer; Len: Word ): Word; override;
       function Process: Word; override;

       procedure OnData( var Data: Pointer ); virtual;
       procedure OnProcess; virtual;

       destructor Destroy; override;
       procedure BeforePlugInit; override;

       procedure DoEvent;

       { For file Transfers. }
       procedure AddTransferFile( FileName: String );
       procedure AddPriorityTransferFile( FileName: String );
       procedure GetTransferList( var List: TStrings );
       function CancelSend( FileName: String ): Integer;
       function CancelReceive: Integer;
       function GetHostPath: String;
       function GetHostFileName: String;
       procedure GetClientFile( FileName: String );

       { Mapped to FOutFileStream events. }
       procedure OnSendDone( Sender: TObject ); dynamic;
       procedure OnSendCancel( Sender: TObject ); dynamic;
       procedure OnSendError( Sender: TObject ); dynamic;
       procedure SendOutFileBlock( var What: TFileBlock ); virtual;

       { Mapped to FInFileStream events.}
       procedure OnReceiveDone( Sender: TObject ); dynamic;
       procedure OnReceiveCancel( Sender: TObject ); dynamic;
       procedure OnReceiveError( Sender: TObject ); dynamic;
       procedure SendInFileBlock( var What: TFileBlock ); virtual;

       property DownLoadDir: String
                read FDownLoadDir
                write FDownLoadDir;
end;



implementation


{ TCustomServer }
function TCustomServer.Command( var Cmd: Pointer; Len: Word ): Word;
begin
 Result := inherited Command( Cmd, Len );
 if Initialized then OnData( Cmd );
end;

function TCustomServer.Process: Word;
begin
 Result := inherited Process;
 if Initialized then OnProcess;
end;

{ These are the preferred methods to do plug processing. }
procedure TCustomServer.OnData( var Data: Pointer );
var Block: TFileBlock;
    RegFile: TAssignFile;
    ListFile: String;
begin
 case PWord(Data)^ of
 
  CHostOutFileBlocks: begin
                   System.Move( Data^, Block, SizeOf( TFileBlock ) );
                   FOutFileStream.Catch( Block );
                  end;

   CHostInFileBlocks: begin
                   System.Move( Data^, Block, SizeOf( TFileBlock ) );
                   FInFileStream.Catch( Block );
                  end;

   CRegisterFile: begin
                    System.Move( Data^, RegFile, SizeOf( TAssignFile ) );
                    FInFileStream.SaveAsName := DownloadDir + ExtractFileName(RegFile.FileName);
                    RegFile.Flag := CSendFile;
                    SendBlock( RegFile, SizeOf( TAssignFile ));
                  end;

    CGetFile: begin
               System.Move( Data^, RegFile, SizeOf( TAssignFile ) );
               if FileExists( DownloadDir + RegFile.FileName ) then
                  AddTransferFile( DownloadDir + ExtractFileName(RegFile.FileName) )
               else
                 begin
                  RegFile.Flag := CInvalidFileName;
                  SendBlock( RegFile, SizeOf( RegFile ) );
                 end;
              end;

    CSendFile: begin
                FOutFileStream.SendFile( FCurrentSend, 1 );
               end;

  end;
end;

procedure TCustomServer.OnProcess;
var RegFile: TAssignFile;
begin
 if (FSendFileReady) and
    (FFileList.Count > 0) then
 begin
  if FileExists( FFileList.Strings[0] ) then
   begin
    FSendFileReady := False;
    FOutFileStream.SaveAsName := FFileList.Strings[0];
    FCurrentSend := FOutFileStream.SaveAsName;
    RegFile.Flag := CRegisterFile;
    RegFile.FileName := ExtractFileName( FFileList.Strings[0] );
    FFileList.Delete(0);
    SendBlock( RegFile, SizeOf( TAssignFile ) );
   end;
 end;
end;

destructor TCustomServer.Destroy;
begin
 if FOutFileStream.SaveAsName <> '' then FOutFileStream.Cancel;
 if FInFileStream.SaveAsName <> '' then FInFileStream.Cancel;
 if Assigned(FFileList) then FFileList.Free;
 if Assigned(FOutFileStream) then FOutFileStream.Free;
 if Assigned(FInFileStream) then FInFileStream.Free;
 inherited Destroy;
end;

procedure TCustomServer.BeforePlugInit;
begin
 inherited BeforePlugInit;
 ClientDllName := '';
 FFileList := TStringList.Create;
 FOutFileStream := TTossFile.Create( CHostOutFileBlocks, CClientInFileBlocks );
 FOutFileStream.OnDone := OnSendDone;
 FOutFileStream.OnCancel := OnSendCancel;
 FOutFileStream.OnError := OnSendError;
 FOutFileStream.OnSend := SendOutFileBlock;
 FInFileStream := TTossFile.Create( CHostInFileBlocks, CClientOutFileBlocks );
 FInFileStream.OnDone := OnReceiveDone;
 FInFileStream.OnCancel := OnReceiveCancel;
 FInFileStream.OnError := OnReceiveError;
 FInFileStream.OnSend := SendInFileBlock;
 FInFileStream.SaveAsName := '';
 FOutFileStream.SaveAsName := '';
 FDownLoadDir := GetHostPath;
 FSendFileReady := True;
end;


procedure TCustomServer.DoEvent;
var Msg: TMsg;
begin
  if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then
  begin
    if Msg.Message <> WM_QUIT then
    begin
     TranslateMessage(Msg);
     DispatchMessage(Msg);
    end
    else Die;
  end;
end;

{ File Transfer Events/Procs. }
 procedure TCustomServer.AddTransferFile( FileName: String );
 begin
  FFileList.Add( DownloadDir + ExtractFileName(FileName) );
 end;

 procedure TCustomServer.AddPriorityTransferFile( FileName: String );
 begin
  if FFileList.Count > 0 then
   FFileList.Insert( 0, FileName )
  else
   FFileList.Add( FileName );
 end;

 procedure TCustomServer.GetTransferList( var List: TStrings );
 begin
  List.Assign( FFileList );
 end;

 function TCustomServer.CancelSend( FileName: String ): Integer;
 var Idx: Integer;
 begin
  Result := CFileIsCanceled;
  if FOutFileStream.SaveAsName = FileName then
     FOutFileStream.Cancel
  else
     begin
      if FFileList.Find( FileName, Idx ) then
         FFileList.Delete( Idx )
      else Result := CFileNotFound;
     end;
 end;

 function TCustomServer.CancelReceive: Integer;
 begin
  Result := CFileIsCanceled;
  if FInFileStream.SaveAsName <> '' then
     FInFileStream.Cancel
  else Result := CFileNotFound;
 end;

 function TCustomServer.GetHostPath: String;
 var chrPath: Array[0..255] of Char;
     Success: Integer;
 begin
  Result := '';
  Success := GetModuleFileName( HInstance, chrPath, 255 );
  if Success > 0 then Result := ExtractFilePath(StrPas( chrPath ));
 end;

 function TCustomServer.GetHostFileName: String;
 var chrPath: Array[0..255] of Char;
     Success: Integer;
 begin
  Result := '';
  Success := GetModuleFileName( HInstance, chrPath, 255 );
  if Success > 0 then Result := StrPas( chrPath );
 end;

 procedure TCustomServer.GetClientFile( FileName: String );
 var ReqFile: TAssignFile;
 begin
  ReqFile.Flag := CGetFile;
  ReqFile.FileName := ExtractFileName( FileName );
  SendBlock( ReqFile, SizeOf( TAssignFile ) );
 end;

 { Mapped to FOutFileStream events. }
 procedure TCustomServer.OnSendDone( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
 end;

 procedure TCustomServer.OnSendCancel( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
 end;

 procedure TCustomServer.OnSendError( Sender: TObject );
 begin
  FOutFileStream.SaveAsName := '';
  FSendFileReady := True;
  { Do nothing. }
 end;

 { Mapped to FInFileStream events.}
 procedure TCustomServer.OnReceiveDone( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
 end;

 procedure TCustomServer.OnReceiveCancel( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
 end;

 procedure TCustomServer.OnReceiveError( Sender: TObject );
 begin
  FInFileStream.SaveAsName := '';
 end;

procedure TCustomServer.SendOutFileBlock( var What: TFileBlock );
begin
 SendBlock( What, SizeOf( TFileBlock ) );
end;

procedure TCustomServer.SendInFileBlock( var What: TFileBlock );
begin
 SendBlock( What, SizeOf( TFileBlock ) );
end;

end.
 