unit Filetoss;

interface

uses WinTypes,
     WinProcs,
     SysUtils,
     Classes;

const MaxBlockSize = 254;

type TTransferState = (tsReceive, tsSend, tsNone );

type TTransferCommand  = ( tcNone, tcRegister, tcReady,
                           tcSend, tcReceive,  tcCancel,
                           tcError, tcDone );

type TProgressEvent = procedure( Sender: TObject;
                                 Total: LongInt;
                                 This: LongInt ) of Object;


type PBlkHeader = ^TBlkHeader;
     TBlkHeader = record
      SysFlag: Word;
      CmdFlag: TTransferCommand;
      TotalSize: LongInt;
      LastOffset: LongInt;
      DataSize: Byte;
end;

type TBlkData = Array[0..(MaxBlockSize-SizeOf(TBlkHeader)-1)] of Char;

type PFileBlock = ^TFileBlock;
     TFileBlock = record
      Header: TBlkHeader;
      Data: TBlkData;
end;

type TFileDoneEvent = procedure ( FileName: String );
type TSendProc = procedure ( var What: TFileBlock ) of Object;

type TAbstractFileMan = class( TObject )
      private
       FCatchBlock: TFileBlock;
       FTossBlock: TFileBlock;
       FSaveAsName: String;
       FSendName: String;
       FCancel: Boolean;
       FState: TTransferState;
       FOnCancel: TNotifyEvent;
       FOnDone: TNotifyEvent;
       FOnError: TNotifyEvent;
       FOnProgress: TProgressEvent;
       FSpeed: Integer;
       FSysInFlag: Word;
       FSysOutFlag: Word;
      protected
       procedure TossBlock; virtual;
       procedure PutBlock( var Block: TFileBlock ); virtual;

       property State: TTransferState
               read FState
               write FState;

      public
       constructor Create( SysInFlag: Word; SysOutFlag: Word );
       function SendFile( FileName: String; Speed: Integer ): Boolean;

       procedure Cancel;
       procedure EndTransfer;

       procedure Send( var What: TFileBlock ); virtual; abstract;
       procedure Catch( var What: TFileBlock ); virtual;

      property SaveAsName: String
               read FSaveAsName
               write FSaveAsName;

      property OnCancel: TNotifyEvent
               read FOnCancel
               write FOnCancel;

      property OnDone: TNotifyEvent
               read FOnDone
               write FOnDone;

      property OnError: TNotifyEvent
               read FOnError
               write FOnError;

      property OnProgress: TProgressEvent
               read FOnProgress
               write FOnProgress;
end;

type TTossFile = class( TAbstractFileMan )
     OnSend: TSendProc;
     procedure Send( var What: TFileBlock ); override;
end;

implementation

constructor TAbstractFileMan.Create( SysInFlag: Word; SysOutFlag: Word );
begin
 inherited Create;
 FSysInFlag := SysInFlag;
 FSysOutFlag := SysOutFlag;
 EndTransfer;
end;

procedure TAbstractFileMan.TossBlock;
var f: File;
    Read: Word;
    Size: LongInt;
    Offset: LongInt;
    Diff: LongInt;
    i: Integer;
begin
 
 for i := 1 to FSpeed do
 begin
 Size := FTossBlock.Header.TotalSize;
 Offset := FTossBlock.Header.LastOffset;
 Diff := Size - Offset;

 if Offset = Size then
  begin
   if Assigned( FOnDone ) then FOnDone( Self );
   if Assigned( FOnProgress ) then FOnProgress( Self , 100, 100 );
   EndTransfer;
   Break;
  end;

 if FState = tsSend then
  begin
   if FileExists( FSendName ) then
    begin
     System.Assign(f, FSendName );
     Reset(f,1);
     Seek(f, Offset );
     FTossBlock.Header.CmdFlag := tcReceive;

     if (Diff >= SizeOf( TBlkData )) then
      begin
       BlockRead( f, FTossBlock.Data, SizeOf( TBlkData ) );
       FTossBlock.Header.DataSize := SizeOf( TBlkData );
       Offset := Offset + SizeOf( TBlkData );
       FTossBlock.Header.LastOffset := Offset;

       if Assigned( FOnProgress ) then FOnProgress( Self, Size, Offset );

       FTossBlock.Header.SysFlag := FSysOutFlag;
       Send( FTossBlock );

       if FTossBlock.Header.LastOffset = Size then
        begin
         FTossBlock.Header.CmdFlag := tcDone;
         FTossBlock.Header.SysFlag := FSysOutFlag;
         Send( FTossBlock );
         EndTransfer;
         if Assigned( FOnDone ) then FOnDone( Self );
        end;
      end;

     if (Diff < SizeOf( TBlkData )) then
      begin
       BlockRead( f, FTossBlock.Data, Diff );
       FTossBlock.Header.DataSize := Diff;
       Offset := Offset + Diff;
       FTossBlock.Header.LastOffset := Offset;
       FTossBlock.Header.SysFlag := FSysOutFlag;

       Send( FTossBlock );

       if Assigned( FOnProgress ) then FOnProgress( Self, 100, 100);

       FTossBlock.Header.CmdFlag := tcDone;
       FTossBlock.Header.DataSize := 0;
       FTossBlock.Header.SysFlag := FSysOutFlag;
       Send( FTossBlock );

       if Assigned( FOnDone ) then FOnDone( Self );

       EndTransfer;
      end;

      System.Close(f);
    end;
   end;
  end;
end;


procedure TAbstractFileMan.PutBlock( var Block: TFileBlock );
var f: File;
    Wrote: Word;
    i: Integer;
begin

 if FState = tsReceive then
  begin
    System.Assign( f, FSaveAsName );
    if FileExists( FSaveAsName ) then
     Reset( f,1 )
    else
     Rewrite( f,1 );

    Seek( f, FileSize(f) );
    BlockWrite( f, Block.Data, Block.Header.DataSize, Wrote );
    System.Close(f);

    Block.Header.CmdFlag := tcSend;
    Block.Header.SysFlag := FSysOutFlag;
    System.Move( Block, FCatchBlock, SizeOf( TFileBlock ) );

    if Assigned( FOnProgress ) then
     FOnProgress( Self, Block.Header.TotalSize, Block.Header.LastOffset );

     Send( Block );

 end;
end;


function TAbstractFileMan.SendFile( FileName: String; Speed: Integer ): Boolean;
var f: File;
begin
 Result := False;
 if FileExists( FileName ) then
  begin
   if FState = tsNone then
    begin
     EndTransfer;
     FState := tsSend;
     FSpeed := Speed;
     FSendName := FileName;

     System.Assign( f, FSendName );
     Reset(f,1);
     FTossBlock.Header.TotalSize := FileSize(f);
     System.Close(f);
     FTossBlock.Header.CmdFlag := tcRegister;
     Send( FTossBlock );
     FTossBlock.Header.CmdFlag := tcReceive;
    end;
  end;
end;


procedure TAbstractFileMan.Cancel;
begin
 FCancel := True;
end;


procedure TAbstractFileMan.EndTransfer;
begin
 with FTossBlock.Header do
  begin
   SysFlag    := FSysOutFlag;
   CmdFlag    := tcNone;
   TotalSize  := 0;
   LastOffset := 0;
   DataSize   := 0;
  end;

 with FCatchBlock.Header do
  begin
   SysFlag    := FSysOutFlag;
   CmdFlag    := tcNone;
   TotalSize  := 0;
   LastOffset := 0;
   DataSize   := 0;
  end;

  FSpeed := 1;
  FState := tsNone;
  FCancel := False;
  FSendName := '';
  FSaveAsName := '';
 end;


procedure TAbstractFileMan.Catch( var What: TFileBlock );
begin

  if FCancel then
    begin
     What.Header.CmdFlag := tcCancel;
     if Assigned( FOnCancel ) then FOnCancel( Self );
     What.Header.SysFlag := FSysOutFlag;
     Send( What );
     EndTransfer;
     Exit;
    end;


 case What.Header.CmdFlag of

     tcReady: TossBlock;
      tcSend: TossBlock;
   tcReceive: PutBlock( What );

  tcRegister: begin
                 if FState <> tsNone then
                  What.Header.CmdFlag := tcError
                 else
                  What.Header.CmdFlag := tcReady;
                  FState := tsReceive;
                  What.Header.SysFlag := FSysOutFlag;
                 Send( What );
              end;

    tcCancel: begin
               if Assigned( FOnCancel ) then
                 FOnCancel( Self );
               EndTransfer;
              end;

     tcError: begin
               if Assigned( FOnError ) then
                 FOnError( Self );
               EndTransfer;
              end;

      tcDone: begin
               if Assigned( FOnDone ) then FOnDone( Self );
               if Assigned( FOnProgress ) then FOnProgress( Self, 100, 100 );
               EndTransfer;
              end;

 end;
end;

procedure TTossFile.Send( var What: TFileBlock );
begin
 if Assigned( OnSend ) then OnSend( What );
end;

end.
