unit Hentry;

{ Delphi Object Programmers Library.
  Copyright 1996 Argon D. Helm.
  Based on source provided by Excalibur Communications Inc.

  These are the entry points for an Excalibur Plugin Camelot Dll.}

interface

uses Dopl,
     WinTypes,
     WinProcs,
     Alchemw,
     SysUtils;


type TPluginClass = class of TPlugin;
const PluginClass: TPluginClass = TPlugin;

{Plug Dll Entry points.}
function  PlugDialog(SlotH: LPSLOT; uResult: WORD):WORD; export;
function  PlugUnique(SlotH: LPSLOT; args, cMessage: PChar):BOOL; export;
procedure PlugDestroy(SlotH: LPSLOT); export;
function  PlugType(SlotH: LPSLOT; title: PChar):WORD; export;
function  PlugProcess(SlotH: LPSLOT):WORD; export;
function  PlugDLLSize(SlotH: LPSLOT):LongInt; export;
function  PlugNextDLL(SlotH: LPSLOT; name: PChar; limit: WORD; dest: PChar;
          dlim: WORD):BOOL; export;
function  PlugDLLVerify(SlotH: LPSLOT; curOS: LongInt; info: Pointer;
          company: PChar; intern: PChar; ver: PChar;
          descript: PChar ):WORD; export;
function  PlugCommand(SlotH: LPSLOT; cCMD: Pointer; cLen: WORD):WORD; export;
function  PlugReceive( SlotH: LPSLOT; data: Pointer; len: Word; sender: LongInt ): Word; export;
function  PlugBroadcast( SlotH: LPSLOT; data: Pointer; len: Word ): Word; export;
function  PlugData(SlotH: LPSLOT; more: BOOL; data: LPVOID):BOOL; export;
procedure PlugPaint( SlotH: LPSLOT; dc: HDC; erase: Bool; rect: PRect ); export;
procedure PlugResize( SlotH: LPVOID; width: Word; height: Word ); export;
function  PlugButton( SlotH: LPSLOT; id: Word ): Word; export;
function  PlugMouse( SlotH: LPSLOT; MouseButton: Byte; x: Word; y: Word ): Word; export;
procedure PlugActivate( SlotH: LPSLOT; active: Bool ); export;


implementation


{Plug Dll Entry points.}
function  PlugType(SlotH: LPSLOT; title: PChar):WORD;
var Plugin: TPlugin;
begin
 Plugin := PluginClass.Create( SlotH );
 Plugin.BeforePlugInit;             {Give user a chance to init properties.}
 Result := Plugin.PlgType( Title ); {Set the title and return plug type}
 Plugin.AlSetPointer( Plugin );     {Save the Plug Object Pointer for future use.}
end;



function  PlugDialog(SlotH: LPSLOT; uResult: WORD):WORD;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Dialog(uResult);
end;



function  PlugUnique(SlotH: LPSLOT; args, cMessage: PChar):BOOL;
var Plugin: TPlugin;
begin
  Plugin := AlGetPointer( SlotH );
  Result := Plugin.Unique( args,cMessage);
end;


procedure PlugDestroy(SlotH: LPSLOT);
var Plugin: TPlugin;
    i: Integer;
begin
 Plugin := AlGetPointer(SlotH);
 Plugin.Free;
end;


function  PlugDLLSize(SlotH: LPSLOT):LongInt;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.DllSize;
end;


function  PlugNextDLL(SlotH: LPSLOT; name: PChar; limit: WORD; dest: PChar;
          dlim: WORD):BOOL;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.NextDll(name, limit, dest, dlim);
end;


function  PlugDLLVerify(SlotH: LPSLOT; curOS: LongInt; info: Pointer;
          company: PChar; intern: PChar; ver: PChar;
          descript: PChar ):WORD;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.CheckDll(curOS,info,company,intern,ver,descript);
end;


function  PlugCommand(SlotH: LPSLOT; cCMD: Pointer; cLen: WORD):WORD;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Command(cCMD, cLen);
end;



function  PlugProcess(SlotH: LPSLOT):WORD;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Process;
end;


function  PlugReceive( SlotH: LPSLOT; data: Pointer; len: Word; sender: LongInt ): Word;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Receive( data, len, sender );
end;


function  PlugBroadcast( SlotH: LPSLOT; data: Pointer; len: Word ): Word;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Broadcast( data, len );
end;


function  PlugData(SlotH: LPSLOT; more: BOOL; data: LPVOID):BOOL;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.GrailData( data,more );
end;


procedure PlugPaint( SlotH: LPSLOT; dc: HDC; erase: Bool; rect: PRect );
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Plugin.Paint(dc,erase,rect);
end;


procedure PlugResize( SlotH: LPVOID; width: Word; height: Word );
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Plugin.Resize(width,height);
end;


function  PlugButton( SlotH: LPSLOT; id: Word ): Word;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Button(id);
end;


function  PlugMouse( SlotH: LPSLOT; MouseButton: Byte; x: Word; y: Word ): Word;
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Result := Plugin.Mouse(MouseButton,x,y);
end;


procedure PlugActivate( SlotH: LPSLOT; active: Bool );
var Plugin: TPlugin;
begin
 Plugin := AlGetPointer(SlotH);
 Plugin.Activate(active);
end;

end.
