unit Main;

 { You may do as you wish with this application
   so long as you dont make any money from it.
   If used for commercial purposes there must
   be at least a 70% change in the source.

   Copyright 1996 Argon D. Helm

   Client Object Programmers Library is not for redistribution
   without expressed permission of the Author.

   Client Object Programmers Library
   Copyright 1996 Argon D. Helm.
 }

interface

uses
  WinTypes,
  WinProcs,
  SysUtils,
  Classes,
  Controls,
  Forms,
  StdCtrls,
  Menus,
  Gauges,
  ExtCtrls,
  Dialogs;

type
  TReadForm = class(TForm)
    mmoReader: TMemo;
    ReadMenu: TMainMenu;
    File1: TMenuItem;
    Save1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Edit1: TMenuItem;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    pnlStatusBar: TPanel;
    gaProgress: TGauge;
    N2: TMenuItem;
    Font1: TMenuItem;
    fd: TFontDialog;
    sd: TSaveDialog;
    pd: TPrintDialog;
    CancelTransfer1: TMenuItem;
    N3: TMenuItem;
    WordWrap1: TMenuItem;
    procedure Cut1Click(Sender: TObject);
    procedure Copy1Click(Sender: TObject);
    procedure Paste1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CancelTransfer1Click(Sender: TObject);
    procedure WordWrap1Click(Sender: TObject);
  private
    { Private declarations }
  public
    ClientHandle: Pointer; {Pointer to TReaderClient's slot handle
                            assigned in the TReaderClient.OnInitialize }
    procedure SetStatus( Value: String );
    procedure OnProgress( Sender: TObject; Total: LongInt; This: LongInt );
    procedure OnFileDone( Sender: TObject; FileName: String; DeleteIt: Boolean );
    procedure OnFileCancel( Sender: TObject; FileName: String );
  end;

var
  ReadForm: TReadForm;

implementation

{$R *.DFM}

uses About,
     Printers,
     Exlib,
     Cread;

procedure TReadForm.SetStatus( Value: String );
const SPACER = #32#32#32#32;
begin
 pnlStatusBar.Caption := SPACER + Value;
end;

procedure TReadForm.Cut1Click(Sender: TObject);
begin
 mmoReader.CutToClipboard;
end;

procedure TReadForm.Copy1Click(Sender: TObject);
begin
 mmoReader.CopyToClipBoard;
end;

procedure TReadForm.Paste1Click(Sender: TObject);
begin
 mmoReader.PasteFromClipboard;
end;

procedure TReadForm.Font1Click(Sender: TObject);
begin
 if fd.Execute then mmoReader.Font := fd.Font;
end;

procedure TReadForm.OnProgress( Sender: TObject; Total: LongInt; This: LongInt );
begin
 if This = Total then
  begin
   gaProgress.Progress := 0;
   CancelTransfer1.Visible := False;
  end
 else
  begin
   if not CancelTransfer1.Visible then
    begin
     CancelTransfer1.Visible := True;
     SetStatus( 'Getting File from BBS...' );
    end;
   if Total > 0 then
      gaProgress.Progress := Trunc((This * 100) / Total);
  end;
end;

procedure TReadForm.OnFileDone( Sender: TObject; FileName: String; DeleteIt: Boolean );
begin
 SetStatus( 'Done...' );
 if FileExists( FileName ) then
  begin
   mmoReader.Lines.LoadFromFile( FileName );
   if DeleteIt then DeleteFile( FileName );
  end;
end;

procedure TReadForm.OnFileCancel( Sender: TObject; FileName: String );
begin
 if FileExists( FileName ) then DeleteFile( FileName );
 ShowMessage( 'Canceled message transfer!' );
 gaProgress.Progress := 0;
end;

procedure TReadForm.Save1Click(Sender: TObject);
begin
 if sd.Execute then
 begin
  DeleteFile( sd.FileName ); {Dont have to exist.}
  mmoReader.Lines.SaveToFile( sd.FileName );
 end;
end;

procedure TReadForm.Exit1Click(Sender: TObject);
begin
 Close;
end;

procedure TReadForm.About1Click(Sender: TObject);
begin
 AboutBox.Show;
end;

procedure TReadForm.Print1Click(Sender: TObject);
var
  I : Integer;
  PrintTxt : TextFile;
  Header : string;
begin
   if pd.Execute then
    begin
      AssignPrn( PrintTxt );
      try
        Rewrite( PrintTxt );
        Printer.Canvas.Font := mmoReader.Font;

        Header := 'Text Reader Version 1.0 --- Date: ' +
                  FormatDateTime( 'ddddd "at" t', Now );

        Writeln( PrintTxt, Header );
        Writeln( PrintTxt );

        for I := 0 to mmoReader.Lines.Count - 1 do
          Writeln( PrintTxt, mmoReader.Lines[ I ] );
      finally
        CloseFile( PrintTxt );
      end;
    end;
end;

procedure TReadForm.FormClose(Sender: TObject; var Action: TCloseAction);
var Client: TReaderClient;
begin
 Action := caNone;
 Client := ExGetPtr( ClientHandle );
 Client.Die;
end;

procedure TReadForm.CancelTransfer1Click(Sender: TObject);
var Client: TReaderClient;
begin
 Client := ExGetPtr( ClientHandle );
 Client.CancelReceive;
end;

procedure TReadForm.WordWrap1Click(Sender: TObject);
begin
 if mmoReader.ScrollBars = ssBoth then
  begin
   WordWrap1.Checked := True;
   mmoReader.ScrollBars := ssVertical;
  end
 else
  begin
   WordWrap1.Checked := False;
   mmoReader.ScrollBars := ssBoth;
  end;
end;

end.
